#!/bin/bash
#version 1.1
set -u

PRODUCT=CPE
PRODUCT_LOWCASE=cpe
upgrade_path=/etc/"$PRODUCT_LOWCASE"agent/upgrade
version_path=/etc/"$PRODUCT_LOWCASE"agent/
conf_file=/etc/${PRODUCT_LOWCASE}agent/${PRODUCT_LOWCASE}agent.conf
bak_conf_file=/etc/${PRODUCT_LOWCASE}agent/${PRODUCT_LOWCASE}agent.conf.upgradebak

logfile="/var/log/update.log"
target="version"
patch_target_type=""
target_name=""
result_id=""

ret_success=0
ret_success_restart=1
ret_success_reboot=2
ret_fail_roolback=6

log()
{
    echo `date` "upgrade:" $@ >> $logfile
}

#write_upgrade_result_conf $1=success/failure
write_upgrade_result_conf()
{
    rm $bak_conf_file -f
    cp $conf_file $bak_conf_file -f
    if [ $? -eq 0 ];then
        sed -i "/upgraderesult/d" $bak_conf_file
        sed -i "/upgradeID/d" $bak_conf_file
        echo "upgraderesult=$1"  >> $bak_conf_file
        echo "upgradeID=${result_id}"  >> $bak_conf_file
        cp $bak_conf_file $conf_file -f
    fi
}

#upgrade_product_version $1=rpm_name
upgrade_product_version()
{
    local deb_name=$1
    log "uninstall $PRODUCT start"
    dpkg -r $PRODUCT >> $logfile 2>&1
    ret=$?
    if [[ $ret -eq 0 ]]; then
        log "uninstall success"
    else
        log "uninstall failed"
        return $ret
    fi
    log "install $PRODUCT $1 start"
    dpkg -i $deb_name>> $logfile 2>&1
    ret=$?
    if [[ $ret -eq 0 ]]; then
        log "install success"
    else
        log "install failed"
    fi
    return $ret
}

upgrade_product_version_rollback()
{
    log "UPGRADE version failed, try ROLLBACK"
    write_upgrade_result_conf "failure"
    deb=`dpkg -l | grep $PRODUCT`
    if [ -z $deb ];then
        for Line in $(ls -lt $upgrade_path | grep $PRODUCT | awk '{print $9}')
        do
            dpkg -i $upgrade_path/$Line >> $logfile
            deb=`dpkg -l| grep $PRODUCT`
            if [ -z $deb ];then
                log "install $Line failed , try again"
            else
                log "ROLLBACK to $Line success"
                log "sytemctl restart $PRODUCT"
                systemctl restart $PRODUCT
                exit
            fi
        done
        log "ROLLBACK failed , anyway exit"
        exit
    else
        log "sytemctl restart $PRODUCT"
        systemctl restart $PRODUCT
        exit 
    fi
}

upgrade_version()
{
    log "----------------$(date)--------------------"
    log "Version upgrade, target=${target_name} result_id=${result_id}"
    if [ ! -f ${upgrade_path}/${target_name} ];then
        log "Could't find $target_name in directory ${upgrade_path}, upgrade ${result_id} failed"
        cpeagent upgradeFailed
        exit
    fi

    upgrade_product_version "${upgrade_path}/${target_name}"

    if [[ $? -eq 0 ]];then
        log "Upgrade ${target_name} upgradeId ${result_id} success"
        write_upgrade_result_conf "success"
        log "systemctl restart $PRODUCT"
        systemctl restart $PRODUCT
        exit
    else
        upgrade_product_version_rollback
    fi
}

manual_version_upgrade()
{
    log "----------------$(date)--------------------"
    log "Manual Version upgrade"
    if [[ ! -d $upgrade_path ]];then
        echo "$upgrade_path dir does not exist"
        log "$upgrade_path dir does not exist"
        exit
    fi
    
    target_name=`ls -lt $upgrade_path | grep -i $PRODUCT |grep deb| head -n1 | awk '{print $9}'`
    if [[ -z $target_name ]];then
        echo "Could not find any upgrade deb in $upgrade_path"
        log "Could not find any upgrade deb in $upgrade_path"
        exit
    fi

    upgrade_product_version "${upgrade_path}/${target_name}"
    if [[ $? -eq 0 ]]; then
        log "Manual Version upgrade success"
        log "systemctl restart $PRODUCT"
        systemctl restart $PRODUCT
    else
        upgrade_product_version_rollback
    fi
}

#handel_patch_return $1=ret
handel_patch_return()
{
    log "Upgrade ${target_name} upgradeId ${result_id} success"
    echo ${target_name##*-} > "${version_path}/PATCH-${patch_target_type}-VERSION"
    local value=$1
    if [ $value -eq 0 ];then
        cpeagent upgradeSuccess
    elif [ $value -eq ${ret_success_restart} ];then
        write_upgrade_result_conf "success"
        log "systemctl restart $PRODUCT"
        systemctl restart $PRODUCT
    elif [ $value -eq ${ret_success_reboot} ];then
        write_upgrade_result_conf "success"
        log "systemctl stop $PRODUCT"
        systemctl stop $PRODUCT
        sync
        log "reboot device"
        reboot -f
    fi
}

#upgrade_product_patch $1=target script
upgrade_product_patch()
{
    local script=$1
    log "install ${target_name} start"
    sh ${script} install >> $logfile 2>&1
    ret=$?
    if [[ $ret -lt $ret_fail_roolback ]]; then
        log "install success $ret"
    else
        log "install failed $ret"
    fi
    return $ret
}

#upgrade_product_patch_rollback $1=target script
patch_rollback()
{
    local script=$1
    log "UPGRADE patch failed, try ROLLBACK"
    log "uninstall ${target_name}"
    sh ${script} uninstall >> $logfile 2>&1
}

get_patch_target_type()
{
    if [[ $target_name =~ THIRD ]];then
        patch_target_type="THIRD"
    elif [[ $target_name =~ CUSTOM ]];then
        patch_target_type="CUSTOM"
    elif [[ $target_name =~ DRIVER ]];then
        patch_target_type="DRIVER"
    fi
}

upgrade_patch()
{
    log "----------------$(date)--------------------"
    log "Patch upgrade, target=${target_name} result_id=${result_id}"

    get_patch_target_type
    if [ -z $patch_target_type ];then
        log "Invalid patch, exit"
        cpeagent upgradeFailed
        exit
    fi

    if [[ ! -d ${upgrade_path}/${target_name} || ! -f ${upgrade_path}/${target_name}/${target_name}.sh ]];then
        log "Could't find $target_name in directory ${upgrade_path}/${target_name}, upgrade ${result_id} failed"
        cpeagent upgradeFailed
    fi

    if [ ! -x "${upgrade_path}/${target_name}/${target_name}.sh" ];then
        chmod 751 ${upgrade_path}/${target_name}/${target_name}.sh
    fi

    upgrade_product_patch "${upgrade_path}/${target_name}/${target_name}.sh"

    ret=$?
    if [[ $ret -lt $ret_fail_roolback ]];then
        handel_patch_return $ret
    else
        log "Upgrade ${target_name} upgradeId ${result_id} failed"
        cpeagent upgradeFailed
        if [ $ret == $ret_fail_roolback ];then
            patch_rollback "${upgrade_path}/${target_name}/${target_name}.sh"
        fi
    fi
}

help()
{
    echo "Usage: $0 --target=<patch/version> --target_name=CPE-3.5.1-20211112.x86_64.rpm --result_id=xxxx"
    echo "      --target/-t         : Target  patch/version"
    echo "      --target_name/-tn   : Upgrade target name, e.g CPE-3.5.1-20211112.x86_64.rpm"
    echo "      --result_id/-r      : upgrade Id"
}

for opt in ${@};
do
    str="="
    result=$(echo $opt | grep "$str")
    if [ -n $result ]; then
        k=${opt%=*}
        v=${opt#*=}
    else
        k=${opt}
    fi

    case ${k} in
        "--target"|"-t") target=${v};;
        "--target_name"|"-tn") target_name=${v};;
        "--result_id"|"-r") result_id=${v};;
        *) echo "${opt} is error";;
    esac
done

if [ $# -lt 3 ];then
    target="manual_version"
else
    [[ -z $target || -z $target_name || -z $result_id ]] && {
        help
        exit 1
    }
fi

if [ ${target} == "patch" ]; then
    upgrade_patch
elif [ ${target} == "version" ]; then
    upgrade_version
elif [ ${target} == "manual_version" ]; then
    echo "manual_version"
    manual_version_upgrade
else
    log "----------------$(date)--------------------"
    log "Unsupported target ${target} result_id=${result_id}"
    cpeagent upgradeFailed
fi
