#!/bin/sh


cpu_num=`cat /proc/cpuinfo | grep "processor" | wc -l` > /dev/null
if [ ${cpu_num} -eq 1 ]; then
    echo "system cpu_num: 1, no need for driver to have fdir"
    exit 0
fi


all_virtual_interfaces=`ls /sys/devices/virtual/net/`

all_interfaces=`ls /sys/class/net/`


is_virtual_inf()
{
	ifname=$1

    is_virt=0
    for virt_interface in $all_virtual_interfaces; do
        if [ "$ifname" == "$virt_interface" ]; then
            is_virt=1
        fi       
    done

    return ${is_virt}
}


#
# ethtool version should have keyword "LW"
#

check_phy_ethx_by_ethtool_version()
{
	ifname=$1

	has_LW=`ethtool -i ${ifname} | grep 'LW' | wc -l`

	if [ ${has_LW} -ge 1 ]; then
		return 1
	else
		return 0
	fi
}


check_phy_ethx_by_proc_dir()
{
	ifname=$1

	driver_name=`ethtool -i $ifname | grep driver | awk '{print $2}'`

    dir=$(find /proc/net/ -type d -name "*_nic_soft_fdir_${driver_name}" | head -n 1)

	if [ -d $dir ]; then
		return 1
	else
		return 0
	fi
}

some_ethx_not_support_fdir=0

check_all_phy_ethx_interface()
{
	for cur_interface in $all_interfaces; do


		#
		# skip virtual interfaces, including bond, br, vlan
		#

		is_virtual_inf ${cur_interface}

		if [ "$?" == "1" ]; then
	
			continue

		fi

		#
		# only check ethX
		#
		
		if [ ${cur_interface:0:3} != "eth" ]; then
		
			continue
		fi

		
		check_phy_ethx_by_ethtool_version ${cur_interface}

		if [ "$?" == "1" ]; then
		
			#
			# it support fdir
			#

			continue

		else

			check_phy_ethx_by_proc_dir ${cur_interface}

			if [ "$?" == "1" ]; then

				#
				# it support fdir
				#

				continue

			else
				
				echo "--- ${cur_interface} does not support FDir"

				ethtool -i ${cur_interface}

				some_ethx_not_support_fdir=1				

			fi

		fi

	done
}

#
# main logic
#

check_all_phy_ethx_interface

if [ ${some_ethx_not_support_fdir} -ne 0 ]; then

	echo "========================================================"

	echo "your system have some ethX not support FDir, please copy all output, and then contact Support and DEV"

	exit 1

else

	echo "all ethX in your system support FDir, good"

	exit 0

fi
