#!/bin/bash

# Locate bc
BC=`which bc`
[ -z "$BC" ] && {
    echo "bc not found, please install \"bc\" using \"yum install bc\" or \"apt-get install bc\" according to your linux distribution"
    exit 1
}

CPUNUM=0
VER_STAGE=1
TOTAL_TIME=65535
CALC_ITV=5 #seconds
HL_START="\033[37;40;1m"
HL_END="\033[0m" 
usermode=0

initCmd() {
    CPUNUM=`ls /proc/net | grep appex | wc -l` 
}

dump_flow()
{

    cmd="$cmd /usr/bin/printf \"total sessions: $HL_START%d$HL_END, \" $NumOfFlows;"
    cmd="$cmd /usr/bin/printf \"tcp sessions: $HL_START%d$HL_END, \" $NumOfTcpFlows;"
    cmd="$cmd /usr/bin/printf \"accelerated tcp sessions: $HL_START%d$HL_END, \" $NumOfAccFlows;"
    cmd="$cmd /usr/bin/printf \"active tcp sessions: $HL_START%d$HL_END, \" $NumOfActFlows;"
    #echo
    cmd="$cmd /usr/bin/printf \"\${showLan:+wan }in: $HL_START%.2F$HL_END kbit/s\t\${showLan:+wan }out: $HL_START%.2F$HL_END kbit/s \" $wanInRate $wanOutRate;"
    [ -n "$showLan" ] && {
        cmd="$cmd /usr/bin/printf \"lan out: $HL_START%.2F$HL_END kbit/s\tlan in:  $HL_START%.2F$HL_END kbit/s \" $lanOutRate $lanInRate;"
        cmd="$cmd /usr/bin/printf \"retransmission ratio: $HL_START%.1F %%$HL_END\" $outRatio;"
    }
    cmd="$cmd /usr/bin/printf \"\n\";"
}

dump_total_flow()
{

    cmd="$cmd /usr/bin/printf \"total sessions: $HL_START%d$HL_END, \" $NumOfFlowsTotal;"
    cmd="$cmd /usr/bin/printf \"tcp sessions: $HL_START%d$HL_END, \" $NumOfTcpFlowsTotal;"
    cmd="$cmd /usr/bin/printf \"accelerated tcp sessions: $HL_START%d$HL_END, \" $NumOfAccFlowsTotal;"
    cmd="$cmd /usr/bin/printf \"active tcp sessions: $HL_START%d$HL_END, \" $NumOfActFlowsTotal;"
    #echo
    cmd="$cmd /usr/bin/printf \"\${showLan:+wan }in :  $HL_START%.2F$HL_END kbit/s\t\${showLan:+wan }out: $HL_START%.2F$HL_END kbit/s\" $wanInRateTotal $wanOutRateTotal;"
    [ -n "$showLan" ] && {
        cmd="$cmd /usr/bin/printf \" lan out: $HL_START%.2F$HL_END kbit/s\tlan in :  $HL_START%.2F$HL_END kbit/s\" $lanOutRateTotal $lanInRateTotal;"
                
        if [ $wanInValTotal -gt 0 ]; then
            outRatio=$(echo "($wanInValTotal - $lanOutValTotal) * 100 / $lanOutValTotal" | bc -l)
        else
            outRatio=0
        fi
        cmd="$cmd /usr/bin/printf \" retransmission ratio: $HL_START%.1F %%$HL_END \" $outRatio;"
    }
    cmd="$cmd /usr/bin/printf \"\n\";"
}

showStats() {
    initCmd
    count=0
    lstTime=$(date +%s)
    cmd=''
    showLan=''
    [ "$1" = "all" ] && showLan=1
    while [ $count -lt $TOTAL_TIME ]; do
        eNum=0
        cmd=''
        wanInRateTotal=0
        lanInRateTotal=0
        wanOutRateTotal=0
        lanOutRateTotal=0
        NumOfFlowsTotal=0
        NumOfTcpFlowsTotal=0
        NumOfAccFlowsTotal=0
        NumOfActFlowsTotal=0
        NfBypassTotal=0
        lanInValTotal=0
        wanOutValTotal=0
        while [ $eNum -lt $CPUNUM ]; do
            if [ $usermode -eq 0 ]; then
                e=$eNum
                [ $e -eq 0 ] && e=''
                [ -d /proc/net/appex$e ] || {
                    ((eNum = $eNum + 1))
                    continue
                }
                eval $(cat /proc/net/appex$e/stats | egrep '(NumOf.*Flows)|(Bytes)|(NfBypass)' |  grep -v Ltt | sed 's/\s*//g')
            else
                eval $($apxexe /$eNum/stats | egrep '(NumOf.*Flows)|(Bytes)|(NfBypass)' | sed 's/\s*//g')
            fi
                
            [ -z "$NumOfFlows" ] && NumOfFlows=0
            [ -z "$NumOfTcpFlows" ] && NumOfTcpFlows=0
            [ -z "$NumOfAccFlows" ] && NumOfAccFlows=0
            [ -z "$NumOfActFlows" ] && NumOfActFlows=0
            [ -z "$WanInBytes" ] && WanInBytes=0
            [ -z "$LanInBytes" ] && LanInBytes=0
            [ -z "$WanOutBytes" ] && WanOutBytes=0
            [ -z "$LanOutBytes" ] && LanOutBytes=0
            [ -z "$NfBypass" ] && NfBypass=0
            
            eval wanInPre=\$wan_in_pre_$e
            eval lanInPre=\$lan_in_pre_$e
            eval wanOutPre=\$wan_out_pre_$e
            eval lanOutPre=\$lan_out_pre_$e
            
            wanIn=$WanInBytes
            lanIn=$LanInBytes
            wanOut=$WanOutBytes
            lanOut=$LanOutBytes
            
            [ -z "$wanInPre" ] && {
                wanInPre=$wanIn
                lanInPre=$lanIn
                wanOutPre=$wanOut
                lanOutPre=$lanOut
            }
            
            eval wan_in_pre_$e=$wanIn
            eval lan_in_pre_$e=$lanIn
            eval wan_out_pre_$e=$wanOut
            eval lan_out_pre_$e=$lanOut

            wanInVal=$(echo "$wanIn - $wanInPre" | bc -l)
            lanInVal=$(echo "$lanIn - $lanInPre" | bc -l)
            wanOutVal=$(echo "$wanOut - $wanOutPre" | bc -l)
            lanOutVal=$(echo "$lanOut - $lanOutPre" | bc -l)
                                    
            #calc ratio
            wanInRate=$(echo "$wanInVal / (128 * $CALC_ITV)" | bc -l)
            lanInRate=$(echo "$lanInVal / (128 * $CALC_ITV)" | bc -l)
            wanOutRate=$(echo "$wanOutVal / (128 * $CALC_ITV)" | bc -l)
            lanOutRate=$(echo "$lanOutVal / (128 * $CALC_ITV)" | bc -l)
            
            # ratio of lanin wanout #(wanout - lanin) / wanout
            if [ $wanInVal -gt 0 ]; then
                outRatio=$(echo "($wanInVal - $lanOutVal) * 100 / $lanOutVal" | bc -l)
            else
                outRatio=0
            fi
            
            if [ $CPUNUM -gt 1 ]; then
                ((NumOfFlowsTotal = $NumOfFlowsTotal + $NumOfFlows))
                ((NumOfTcpFlowsTotal = $NumOfTcpFlowsTotal + $NumOfTcpFlows))
                ((NumOfAccFlowsTotal = $NumOfAccFlowsTotal + $NumOfAccFlows))
                ((NumOfActFlowsTotal = $NumOfActFlowsTotal + $NumOfActFlows))
                wanInRateTotal=$(echo "$wanInRateTotal + $wanInRate" | bc -l)
                lanInRateTotal=$(echo "$lanInRateTotal + $lanInRate" | bc -l)
                wanOutRateTotal=$(echo "$wanOutRateTotal + $wanOutRate" | bc -l)
                lanOutRateTotal=$(echo "$lanOutRateTotal + $lanOutRate" | bc -l)
                ((NfBypassTotal = $NfBypassTotal + $NfBypass))
                ((wanInValTotal = $wanInValTotal + $wanInVal))
                ((wanOutValTotal = $wanOutValTotal + $wanOutVal))
                ((lanInValTotal = $lanInValTotal + $lanInVal))
                ((lanOutValTotal = $lanOutValTotal + $lanOutVal))
            fi
            
            cmd="$cmd echo \"engine#$eNum:\";"
            dump_flow
            cmd="$cmd /usr/bin/printf \"\n\";"
            ((eNum = $eNum + 1))
            ((NfBypass = 0))
        done
        if [ $CPUNUM -gt 1 ]; then
            cmd="$cmd echo \"Total:\";"
            dump_total_flow
            cmd="$cmd /usr/bin/printf \"\n\";"
            echo
        fi
        clear
        eval $cmd
        sleep $CALC_ITV
        ((count = $count + $CALC_ITV))
    done
}
if [ $# -gt 0 ];then
    CALC_ITV=$1
fi
showStats $2


