#!/bin/bash
# author   : Yin ZhiHao
# version  : 0.01
# history  : 12/08/2022 v0.01  init draft


# write log threshold
declare -i MEM_THRESHOLD_1=80
declare -i MEM_THRESHOLD_2=90
declare -i MEM_THRESHOLD_3=95

# dingding warning remind

# DingTalk intelligent group assistant custom bot to get webhooks
# webhook=https://oapi.dingtalk.com/robot/send?access_token=111111

site_name=`cat /etc/motd | sed '/^$/d'`
send_message()
{
    time_now=$(date "+%Y-%m-%d %H:%M:%S")
    curl  $webhook -H 'Content-Type: application/json' -d "
    {
        'msgtype': 'text',
        'text': {
            'content': '${1}\n Alarm:\n${site_name}\n Time: ${time_now}\n \n'
        },
        'at': {
            'isAtAll': true
        }
    }"
}

# parse /proc/meminfo and set the relative parameters
get_meminfo()
{
    MemTotal=`cat /proc/meminfo | grep -w MemTotal | awk '{print $2}'`
    MemFree=`cat /proc/meminfo | grep -w MemFree | awk '{print $2}'`
    MemAvailable=`cat /proc/meminfo | grep -w MemAvailable | awk '{print $2}'`
    Buffers=`cat /proc/meminfo | grep -w Buffers | awk '{print $2}'`
    Cached=`cat /proc/meminfo | grep -w Cached | awk '{print $2}'`

    Active=`cat /proc/meminfo | grep -w Active: | awk '{print $2}'`
    Inactive=`cat /proc/meminfo | grep -w Inactive: | awk '{print $2}'`

    Unevictable=`cat /proc/meminfo | grep -w Unevictable | awk '{print $2}'`

    Slab=`cat /proc/meminfo | grep -w Slab | awk '{print $2}'`

    KernelStack=`cat /proc/meminfo | grep -w KernelStack | awk '{print $2}'`
    PageTables=`cat /proc/meminfo | grep -w PageTables | awk '{print $2}'`
    Bounce=`cat /proc/meminfo | grep -w Bounce | awk '{print $2}'`

    VmallocUsed=`cat /proc/meminfo | grep -w VmallocUsed | awk '{print $2}'`

    HardwareCorrupted=`cat /proc/meminfo | grep -w HardwareCorrupted | awk '{print $2}'`

    HugePages_Total=`cat /proc/meminfo | grep -w HugePages_Total | awk '{print $2}'`
    Hugepagesize=`cat /proc/meminfo | grep -w Hugepagesize | awk '{print $2}'`

    HardwareCorrupted=`cat /proc/meminfo | grep -w HardwareCorrupted | awk '{print $2}'`
}

nic_statistics()
{
    for i in 0 1 2 3 4 5; do
        op="ethtool -S eth$i"
        cue="eth$i statistics:"
        nicsta_list=`ethtool -S eth$i | sed 1d`
        echo -e ""$op"\n"$cue"\n${nicsta_list}\n" >> $path/nic_stats.log
    done
}

query_congestion()
{
    op="cat /sys/kernel/debug/qman/query_congestion"
    congest=`cat /sys/kernel/debug/qman/query_congestion`

    echo -e ""$op"\n${congest}" >> $path/congestion.log
}

collect_regs()
{
    index=1
    
    for ((i=0x1a88000, j=0x1aa8000; i<=0x1a8d000, j<=0x1aad000; i+=0x1000,j+=0x1000))
    do
        echo "MAC$index " >> $path/devices_regs.log
        index=`expr $index + 1`

        rxaddr=`echo "$i" | awk '{printf("%x",$1)}'`
        txaddr=`echo "$j" | awk '{printf("%x",$1)}'`
        op1="cat /sys/devices/platform/soc/1a00000.fman/$rxaddr.port/fm_port_bmi_regs"
        op2="cat /sys/devices/platform/soc/1a00000.fman/$txaddr.port/fm_port_qmi_regs"
        bmi_regs=`cat /sys/devices/platform/soc/1a00000.fman/$rxaddr.port/fm_port_bmi_regs`
        qmi_regs=`cat /sys/devices/platform/soc/1a00000.fman/$txaddr.port/fm_port_qmi_regs`

        echo -e ""$op1"\n$bmi_regs\n\n"$op2"\n$qmi_regs\n" >> $path/devices_regs.log
    done
}

EMAC_statistics()
{
    for i in 0 1 2 3 4 5; do
        echo "  " >> $path/EMAC_stats.log
        op1="cat /sys/class/net/eth$i/mac_regs"
        op2="cat /sys/class/net/eth$i/mac_rx_stats"
        op3="cat /sys/class/net/eth$i/mac_tx_stats"
        mac_regs=`cat /sys/class/net/eth$i/mac_regs`
        mac_rx_stats=`cat /sys/class/net/eth$i/mac_rx_stats`
        mac_tx_stats=`cat /sys/class/net/eth$i/mac_tx_stats`

        echo -e ""$op1"\n${mac_regs}\n\n"$op2"\n${mac_rx_stats}\n\n"$op3"\n${mac_tx_stats}\n" >> $path/EMAC_stats.log
    done
}


date_log()
{
    DATE=`date '+%Y%m%d-%H%M%S'`

    echo "——————————————————————————————————————————————————————" >> $2
    echo "memory usages $1%!   BACKUP DATE:" $(date +"%Y-%m-%d %H:%M:%S") >> $2
    content="memory usages $1%! $(date +"%Y-%m-%d %H:%M:%S") write to the log: $2"
    echo -e "\033[31m" ${content} "\033[0m"
    echo "——————————————————————————————————————————————————————" >> $2
}

write_log()
{
    
    mkdir  $LOG_PATH/dump_info$time
    path=$LOG_PATH/dump_info$time

    date_log $1 $path/nic_stats.log
    date_log $1 $path/congestion.log
    date_log $1 $path/EMAC_stats.log
    date_log $1 $path/devices_regs.log
    
    nic_statistics
    query_congestion
    EMAC_statistics
    collect_regs
}


#### main process ####

if [ ! -d "/var/log/NIC_failedInfo" ]; then
    mkdir /var/log/NIC_failedInfo
fi

LOG_PATH=/var/log/NIC_failedInfo

old_perent=`echo $i | awk '{printf"%d", 0 }'`

# write flag
threshold1_write=0
threshold2_write=0
threshold3_write=0

while ((1))
do
    time=`date '+%Y%m%d-%H%M%S'`

    get_meminfo

    if [ $VmallocUsed -eq 0 ]; then
        old=$IFS
        IFS=$'\n'
        vmallc_list=`cat /proc/vmallocinfo`
        for i in $vmallc_list; do
            vmem=`echo $i | awk '{print $2}'`
            VmallocUsed=$(($VmallocUsed+$vmem))
        done
        IFS=$old
    fi
    usedMem=`expr $MemTotal - $MemFree`
    kernelMem=`expr $Slab + $VmallocUsed + $PageTables + $KernelStack + $HardwareCorrupted + $Bounce`
    userMem=`expr $Active + $Inactive + $Unevictable`
    hugeMem=`expr $HugePages_Total \* $Hugepagesize`

    # check memory usage
    mem_usage_percent=$(echo "scale=2; (1 - $MemAvailable / $MemTotal) * 100" | bc)

    tmp1=`echo "$mem_usage_percent $MEM_THRESHOLD_1" | awk '{if($1<$2) {printf"%d",1} else {printf"%d",0}}'`
    tmp2=`echo "$mem_usage_percent $MEM_THRESHOLD_2" | awk '{if($1>=$2) {printf"%d",1} else {printf"%d",0}}'`
    tmp3=`echo "$mem_usage_percent $MEM_THRESHOLD_3" | awk '{if($1<$2) {printf"%d",1} else {printf"%d",0}}'`


    if [ $tmp1 -eq 1 ]; then
        # memory usages less than 80
        percent=`echo "$mem_usage_percent" | awk '{printf("%d",$1)}'`
        if [ $percent -ne $old_perent ]; then
            echo "memory usages $mem_usage_percent%, no need to diagnose"
            old_perent=`echo "$mem_usage_percent" | awk '{printf("%d",$1)}'`
        fi
    elif [[ $tmp1 -eq 0 && $tmp2 -eq 0 ]]; then
        # memory usages more than 80 less than 90
        if [ $threshold1_write -eq 0 ]; then
            content="memory usages $mem_usage_percent%! Log information is located at /var/log/NIC_failedInfo"
            echo -e "\033[33m" ${content} "\033[0m"
            write_log 80 $time
            send_message "memory usages $mem_usage_percent%!"
            threshold1_write=1
        fi
    elif [[ $tmp2 -eq 1 && $tmp3 -eq 1 ]]; then
        # memory usages more than 90 less than 95
        if [ $threshold2_write -eq 0 ]; then
            content="memory usages $mem_usage_percent%! Log information is located at /var/log/NIC_failedInfo "
            echo -e "\033[33m" ${content} "\033[0m"
            write_log 90 $time
            send_message "memory usages $mem_usage_percent%!"
            threshold2_write=1
        fi      
    else
        # memory usages more than 95
        if [ $threshold3_write -eq 0 ]; then
            content="memory usages $mem_usage_percent%! Log information is located at /var/log/NIC_failedInfo"
            echo -e "\033[33m" ${content} "\033[0m"
            write_log 95 $time
            send_message "memory usages $mem_usage_percent%!"
            threshold3_write=1
        fi     
    fi

    # detection frequency one second
    sleep 1

done
