#!/bin/bash
#v2.0
#2021-01-15

#time
CURRENTTIME=$(date "+%Y-%m-%d_%H%M")

#path
COLLECT_PATH=/var/log/monitor/collect_$CURRENTTIME
CONF_PATH=$COLLECT_PATH/conf
INFO_PATH=$COLLECT_PATH/info
LOG_PATH=$COLLECT_PATH/log

#info
NET_INFO_FILE=$INFO_PATH/network_info_${CURRENTTIME}.log
SYS_INFO_FILE=$INFO_PATH/sys_info_${CURRENTTIME}.log
PROCESS_INFO_FILE=$INFO_PATH/process_info_${CURRENTTIME}.log
IPTABLES_INFO_FILE=$INFO_PATH/iptables_info_${CURRENTTIME}.log
LW_INFO_FILE=$INFO_PATH/lw_info_${CURRENTTIME}.log
DEV_INFO_FILE=$INFO_PATH/dev_info_${CURRENTTIME}.log
INTPT_INFO_FILE=$INFO_PATH/interrupts_info_${CURRENTTIME}.log
LSPCI_INFO_FILE=$INFO_PATH/lspci_info_${CURRENTTIME}.log
CPU_INFO_FILE=$INFO_PATH/cpu_info_${CURRENTTIME}.log

#product flag
CPE_FLAG=0
POP_FLAG=0

#cpu usage
declare -a cpu_parameters_name
declare -a cpu_parameters_previous
declare -a cpu_parameters_now
declare -a cpu_D_value
declare -a cpu_previous_total_time
declare -a cpu_now_total_time
declare -a cpu_total_time_D_value
declare -a cpu_previous_idle_time
declare -a cpu_now_idle_time
declare -a cpu_idle_time_D_value

lw_conf_collect(){
    if [ ! -d $CONF_PATH ]; then
        mkdir -p $CONF_PATH
    fi
    
    cp /etc/appex/*.conf $CONF_PATH
    cp /etc/appex/config $CONF_PATH
    cp ${agent_path}/*.conf $CONF_PATH -rf
    cp ${agent_path}/RPM_version $CONF_PATH -rf
    cp ${agent_path}/ModConfig $CONF_PATH -rf

}

dev_info_collect(){
    if [ ! -d $INFO_PATH ]; then
        mkdir -p $INFO_PATH
    fi
    
    echo -------------dev info `date "+%Y-%m-%d_%H:%M:%S"`-------------- >> $DEV_INFO_FILE
    cat /proc/net/dev >> $DEV_INFO_FILE 2>&1
}


#cpu_usage_get_time $1=total $2=user $3=nice $4=system $5=idle $6=iowait $7=irq $8=softirq
cpu_usage_get_time(){
    unset $1
    unset $2
    unset $3
    unset $4
    unset $5
    unset $6
    unset $7
    unset $8
    for((i=0;i<${cpu_total_num};i++));do
        stat=`cat /proc/stat | grep "cpu$i" | awk '{print $2" "$3" "$4" "$5" "$6" "$7" "$8}'`
        eval $1[$i]=`echo ${stat} | awk '{printf "%.f",$1+$2+$3+$4+$5+$6+$7}'`
        eval $2[$i]=`echo ${stat} | awk '{print $1}'`
        eval $3[$i]=`echo ${stat} | awk '{print $2}'`
        eval $4[$i]=`echo ${stat} | awk '{print $3}'`
        eval $5[$i]=`echo ${stat} | awk '{print $4}'`
        eval $6[$i]=`echo ${stat} | awk '{print $5}'`
        eval $7[$i]=`echo ${stat} | awk '{print $6}'`
        eval $8[$i]=`echo ${stat} | awk '{print $7}'`
    done
}

lspci_info_collect(){
    if [ ! -d $INFO_PATH ]; then
        mkdir -p $INFO_PATH
    fi
  
    echo -------------lspci-------------- >> $LSPCI_INFO_FILE
    lspci -vvv >> $LSPCI_INFO_FILE 2>&1
    
}

eth_info_first_collect(){
    if [ ! -d $INFO_PATH ]; then
        mkdir -p $INFO_PATH
    fi

    for net_device in `ls /sys/class/net/ | grep -v 've\|vs\|brlan[3-9][0-9]'`;do
        ETHINFOFILE=$INFO_PATH/eth_info_${net_device}_${CURRENTTIME}.log

        echo -------------ethtool ${net_device}-------------- >> $ETHINFOFILE
        ethtool ${net_device}>> $ETHINFOFILE 2>&1

        echo -------------ethtool -i ${net_device}-------------- >> $ETHINFOFILE
        ethtool -i ${net_device}>> $ETHINFOFILE 2>&1

        bus_info=`ethtool -i ${net_device} | grep bus-info | awk '{print $2}'`
        echo -------------/sys/bus/pci/devices/${bus_info}/enable-------------- >> $ETHINFOFILE
        cat /sys/bus/pci/devices/${bus_info}/enable >> $ETHINFOFILE 2>&1

        echo -------------/sys/bus/pci/devices/${bus_info}/net/${net_device}/carrier-------------- >> $ETHINFOFILE
        cat /sys/bus/pci/devices/${bus_info}/net/${net_device}/carrier >> $ETHINFOFILE 2>&1

        echo -------------/sys/bus/pci/devices/${bus_info}/net/${net_device}/carrier_changes-------------- >> $ETHINFOFILE
        cat /sys/bus/pci/devices/${bus_info}/net/${net_device}/carrier_changes >> $ETHINFOFILE 2>&1

        echo -------------/sys/bus/pci/devices/${bus_info}/net/${net_device}/link_mode -------------- >> $ETHINFOFILE
        cat /sys/bus/pci/devices/${bus_info}/net/${net_device}/link_mode  >> $ETHINFOFILE 2>&1

        echo -------------/sys/bus/pci/devices/${bus_info}/net/${net_device}/operstate  -------------- >> $ETHINFOFILE
        cat /sys/bus/pci/devices/${bus_info}/net/${net_device}/operstate   >> $ETHINFOFILE 2>&1

        echo -------------/sys/bus/pci/devices/${bus_info}/net/${net_device}/flags   -------------- >> $ETHINFOFILE
        cat /sys/bus/pci/devices/${bus_info}/net/${net_device}/flags    >> $ETHINFOFILE 2>&1

        driver_name=`ethtool -i ${net_device} | awk '{print $2}' | sed -n 1p`> /dev/null
        echo -------------modeinfo ${driver_name}-------------- >> $ETHINFOFILE
        modinfo ${driver_name}>> $ETHINFOFILE 2>&1

        echo -------------ethtool -k ${net_device}-------------- >> $ETHINFOFILE
        ethtool -k ${net_device}>> $ETHINFOFILE 2>&1

        echo -------------ethtool -g ${net_device}-------------- >> $ETHINFOFILE
        ethtool -g ${net_device}>> $ETHINFOFILE 2>&1

        echo -------------ethtool -S ${net_device} `date "+%Y-%m-%d_%H:%M:%S"`-------------- >> $ETHINFOFILE
        ethtool -S ${net_device}>> $ETHINFOFILE 2>&1

        echo -------------tc -s qdisc show dev ${net_device} `date "+%Y-%m-%d_%H:%M:%S"`-------------- >> $ETHINFOFILE
        tc -s qdisc show dev ${net_device}>> $ETHINFOFILE 2>&1

    done
}

eth_info_second_collect(){
    if [ ! -d $INFO_PATH ]; then
        mkdir -p $INFO_PATH
    fi

    for net_device in `ls /sys/class/net/ | grep -v 've\|vs\|brlan[3-9][0-9]'`;do

        ETHINFOFILE=$INFO_PATH/eth_info_${net_device}_${CURRENTTIME}.log

        echo -------------ethtool -S ${net_device} `date "+%Y-%m-%d_%H:%M:%S"`-------------- >> $ETHINFOFILE
        ethtool -S ${net_device}>> $ETHINFOFILE 2>&1

        echo -------------tc -s qdisc show dev ${net_device} `date "+%Y-%m-%d_%H:%M:%S"`-------------- >> $ETHINFOFILE
        tc -s qdisc show dev ${net_device}>> $ETHINFOFILE 2>&1
    done
}

network_info_collect(){
    if [ ! -d $INFO_PATH ]; then
        mkdir -p $INFO_PATH
    fi

    echo ============================== >> $NET_INFO_FILE
    echo -------------ifconfig-------------- >> $NET_INFO_FILE 
    ifconfig -a >> $NET_INFO_FILE 2>&1
    echo -------------ip addr-------------- >> $NET_INFO_FILE
    ip addr >> $NET_INFO_FILE 2>&1
    echo -------------ip rule-------------- >> $NET_INFO_FILE
    ip rule >> $NET_INFO_FILE 2>&1
    echo -------------ip route-------------- >> $NET_INFO_FILE
    ip route >> $NET_INFO_FILE 2>&1
    echo -------------dns rule-------------- >> $NET_INFO_FILE
    cat /etc/resolv.conf >> $NET_INFO_FILE 2>&1
    echo ----------netdev_max_backlog--------- >> $NET_INFO_FILE
    cat /proc/sys/net/core/netdev_max_backlog >> $NET_INFO_FILE 2>&1
    echo ----------netdev_budget--------- >> $NET_INFO_FILE
    cat /proc/sys/net/core/netdev_budget >> $NET_INFO_FILE 2>&1
    INFO_FLAG=1
}

engine_info_collect(){
    if [ -d /proc/net/appex ]; then
        echo ============================== >> $LW_INFO_FILE
        for appex in `ls /proc/net/appex* -d`
        do
            echo -------------------$appex/cmd ------------->>$LW_INFO_FILE
            cat $appex/cmd >> $LW_INFO_FILE
            echo -------------------`date "+%Y-%m-%d_%H:%M:%S"`$appex/stats------------->>$LW_INFO_FILE
            cat $appex/stats >> $LW_INFO_FILE
            sleep 2
            echo -------------------`date "+%Y-%m-%d_%H:%M:%S"`$appex/stats------------->>$LW_INFO_FILE
            cat $appex/stats >> $LW_INFO_FILE
        done
    else
        echo ============================== >> $LW_INFO_FILE
        echo **********/proc/net/appex directory does not exist********** >> $LW_INFO_FILE
    fi
}

lw_info_collect(){
    if [ ! -d $INFO_PATH ]; then
        mkdir -p $INFO_PATH
    fi
    
    echo ============================== >> $LW_INFO_FILE
    echo -------------agent version-------------- >> $LW_INFO_FILE
    ${agent} -v >> $LW_INFO_FILE 2>&1
    echo -------------if-tbl-------------- >> $LW_INFO_FILE
    cat /proc/net/lw-conf/if-tbl >> $LW_INFO_FILE 2>&1
    echo -------------lan-tbl-------------- >> $LW_INFO_FILE
    cat /proc/net/lw-conf/lan-tbl >> $LW_INFO_FILE 2>&1
    echo -------------wan-tbl-------------- >> $LW_INFO_FILE
    cat /proc/net/lw-conf/wan-tbl >> $LW_INFO_FILE 2>&1
    echo -------------policy_table-------------- >> $LW_INFO_FILE
    cat /proc/net/lw-conf/policy_table >> $LW_INFO_FILE 2>&1
    echo -------------flow_table-------------- >> $LW_INFO_FILE
    cat /proc/net/lw-conf/flow_table >> $LW_INFO_FILE 2>&1
    echo -------------wan-stats-------------- >> $LW_INFO_FILE
    cat /proc/net/lw-conf/wan-stats >> $LW_INFO_FILE 2>&1
    echo -------------mempool-usage-------------- >> $LW_INFO_FILE
    cat /proc/net/lw-conf/mempoolusage >> $LW_INFO_FILE 2>&1
    echo -------------mem-usage-------------- >> $LW_INFO_FILE
    cat /proc/net/lw-conf/memusage >> $LW_INFO_FILE 2>&1
    echo -------------lw_stats-------------- >> $LW_INFO_FILE
    cat /proc/net/lw-conf/lw_stats >> $LW_INFO_FILE 2>&1
    echo -------------ltt/stats-------------- >> $LW_INFO_FILE
    cat /proc/net/lw-conf/ltt/stats >> $LW_INFO_FILE 2>&1
    echo -------------engine-version-------------- >> $LW_INFO_FILE
    cat /proc/net/appex/version  >> $LW_INFO_FILE 2>&1

    engine_info_collect
}

cpu_usage_get_parameters(){
    unset $1
    unset $2
    unset $3
    for i in $(seq 1 ${cpu_parameter_counts});do
        cpu_parameter_row=`expr ${i} / ${cpu_columns} + 1`
        cpu_parameter_column=`expr ${i} % ${cpu_columns}`
        if [ $cpu_parameter_column -eq 0 ];then
            cpu_parameter_column=${cpu_columns}
        fi

        if [ $cpu_parameter_column -eq ${cpu_columns} ];then
            cpu_parameter_row=`expr ${cpu_parameter_row} - 1`
        fi

        tmp_value=`cat /proc/stat | grep cpu | awk '{print $'$cpu_parameter_column'}'|sed 's/://g' | sed -n ${cpu_parameter_row}p`> /dev/null
        eval $1[i]=$tmp_value
        if [ $cpu_parameter_column -gt 1 ];then
            eval let $2[$cpu_parameter_row]+=$tmp_value
        fi
        if [ $cpu_parameter_column -eq 5 ];then
            eval $3[$cpu_parameter_row]=$tmp_value
        fi
    done
}

cpu_usage_first_collect(){
    if [ ! -d $INFO_PATH ]; then
        mkdir -p $INFO_PATH
    fi
    
    echo ----------proc/stat `date "+%Y-%m-%d_%H:%M:%S"`--------- >> $CPU_INFO_FILE
    cat /proc/stat | grep cpu >> $CPU_INFO_FILE 2>&1

    cpu_total_num=`cat /proc/stat | grep cpu[0-99] -c`
    cpu_usage_get_time cpu_previous_total_time cpu_previous_user_time cpu_previous_nice_time cpu_previous_system_time cpu_previous_idle_time cpu_previous_iowait_time cpu_previous_irq_time cpu_previous_softirq_time
}

cpu_usage_second_collect(){
    if [ ! -d $INFO_PATH ]; then
        mkdir -p $INFO_PATH
    fi

    echo ----------proc/stat `date "+%Y-%m-%d_%H:%M:%S"`--------- >> $CPU_INFO_FILE
    cat /proc/stat | grep cpu >> $CPU_INFO_FILE 2>&1

    cpu_usage_get_time cpu_now_total_time cpu_now_user_time cpu_now_nice_time cpu_now_system_time cpu_now_idle_time cpu_now_iowait_time cpu_now_irq_time cpu_now_softirq_time

    for((i=0;i<${cpu_total_num};i++));do
        cpu_num=$i
        cpu_total_time_D_value=`expr ${cpu_now_total_time[i]} - ${cpu_previous_total_time[i]}`
        cpu_idle_time_D_value=`expr ${cpu_now_idle_time[i]} - ${cpu_previous_idle_time[i]}`
        #get cpu usage percentage 
        cpu_usage_percent=`echo "$cpu_idle_time_D_value $cpu_total_time_D_value"| awk '{printf "%.2f\n",(1-$1/$2)*100}'`

        cpu_user_time_D_value=`expr ${cpu_now_user_time[i]} - ${cpu_previous_user_time[i]}`
        cpu_nice_time_D_value=`expr ${cpu_now_nice_time[i]} - ${cpu_previous_nice_time[i]}`
        cpu_system_time_D_value=`expr ${cpu_now_system_time[i]} - ${cpu_previous_system_time[i]}`
        cpu_iowait_time_D_value=`expr ${cpu_now_iowait_time[i]} - ${cpu_previous_iowait_time[i]}`
        cpu_irq_time_D_value=`expr ${cpu_now_irq_time[i]} - ${cpu_previous_irq_time[i]}`
        cpu_softirq_time_D_value=`expr ${cpu_now_softirq_time[i]} - ${cpu_previous_softirq_time[i]}`
        cpu_user_percent=`echo "$cpu_user_time_D_value $cpu_total_time_D_value"| awk '{printf "%.2f\n",($1/$2)*100}'`
        cpu_user_percent=`echo "$cpu_user_time_D_value $cpu_total_time_D_value"| awk '{printf "%.2f\n",($1/$2)*100}'`
        cpu_nice_percent=`echo "$cpu_nice_time_D_value $cpu_total_time_D_value"| awk '{printf "%.2f\n",($1/$2)*100}'`
        cpu_system_percent=`echo "$cpu_system_time_D_value $cpu_total_time_D_value"| awk '{printf "%.2f\n",($1/$2)*100}'`
        cpu_iowait_percent=`echo "$cpu_iowait_time_D_value $cpu_total_time_D_value"| awk '{printf "%.2f\n",($1/$2)*100}'`
        cpu_irq_percent=`echo "$cpu_irq_time_D_value $cpu_total_time_D_value"| awk '{printf "%.2f\n",($1/$2)*100}'`
        cpu_softirq_percent=`echo "$cpu_softirq_time_D_value $cpu_total_time_D_value"| awk '{printf "%.2f\n",($1/$2)*100}'`
        echo "----------------------cpu_usage-----------------------------" >> $CPU_INFO_FILE
        echo "cpu$cpu_num usage[${cpu_usage_percent}%] user[${cpu_user_percent}%] nice[${cpu_nice_percent}%] system[${cpu_system_percent}%] iowait[${cpu_iowait_percent}%] irq[${cpu_irq_percent}%] softirq[${cpu_softirq_percent}%]" >> $CPU_INFO_FILE
    done
}

interrupts_info_collect(){
    if [ ! -d $INFO_PATH ]; then
        mkdir -p $INFO_PATH
    fi

    echo ----------proc/interrupts `date "+%Y-%m-%d_%H:%M:%S"`--------- >> $INTPT_INFO_FILE
    cat /proc/interrupts >> $INTPT_INFO_FILE 2>&1
}

sys_info_collect(){
    if [ ! -d $INFO_PATH ]; then
        mkdir -p $INFO_PATH
    fi
    echo ============================== >> $SYS_INFO_FILE
    echo -----------version------------ >> $SYS_INFO_FILE
    cat /proc/version >> $SYS_INFO_FILE 2>&1
    uname -a>> $SYS_INFO_FILE 2>&1
    echo -------------free-------------- >> $SYS_INFO_FILE
    free >> $SYS_INFO_FILE 2>&1
    echo -----------meminfo------------ >> $SYS_INFO_FILE
    cat /proc/meminfo >> $SYS_INFO_FILE 2>&1
    echo -----------slab info---------- >> $SYS_INFO_FILE
    cat /proc/slabinfo >> $SYS_INFO_FILE 2>&1
    echo --------slab cache info------- >> $SYS_INFO_FILE
    slabtop -o >> $SYS_INFO_FILE 2>&1
    echo --------proc/vmallocinfo------ >> $SYS_INFO_FILE
    cat /proc/vmallocinfo >> $SYS_INFO_FILE 2>&1
    echo ----------proc/vmstat--------- >> $SYS_INFO_FILE
    cat /proc/vmstat >> $SYS_INFO_FILE 2>&1
}

process_info_collect(){
    if [ ! -d $INFO_PATH ]; then
        mkdir -p $INFO_PATH
    fi
    echo ============================== >> $PROCESS_INFO_FILE

    echo -------------cpu usage-------------- >> $PROCESS_INFO_FILE
    mpstat -P ALL >> $PROCESS_INFO_FILE 2>&1

    echo -------------top-------------- >> $PROCESS_INFO_FILE
    top -n 1 -b >> $PROCESS_INFO_FILE 2>&1
    echo -------------ps aux-------------- >> $PROCESS_INFO_FILE
    ps aux >> $PROCESS_INFO_FILE 2>&1
    echo -------------lsof-------------- >> $PROCESS_INFO_FILE
    lsof  >> $PROCESS_INFO_FILE 2>&1

}

iptables_info_collect(){
    if [ ! -d $INFO_PATH ]; then
        mkdir -p $INFO_PATH
    fi
    echo ============================== >> $IPTABLES_INFO_FILE
    echo -------------firewall stats-------------- >> $IPTABLES_INFO_FILE
    systemctl status firewalld >> $IPTABLES_INFO_FILE 2>&1
    echo -------------iptables stats-------------- >> $IPTABLES_INFO_FILE
    iptables -nvL >> $IPTABLES_INFO_FILE 2>&1
    echo -------------iptables configuration-------------- >> $IPTABLES_INFO_FILE
    iptables -t mangle -nvL >> $IPTABLES_INFO_FILE 2>&1
    echo -------------iptables rule-------------- >> $IPTABLES_INFO_FILE
    iptables -t nat -nvL >> $IPTABLES_INFO_FILE 2>&1
}

log_collect(){
    if [ ! -d $LOG_PATH ]; then
        mkdir -p $LOG_PATH
    fi
    find /var/log/ -maxdepth 1 -type f -name "messages*" | xargs ls -ta | head -n 2 | xargs -i cp {} $LOG_PATH
    find /var/log/ -maxdepth 1 -type f -name "$agent_log*" | xargs ls -ta | head -n 2 | xargs -i cp {} $LOG_PATH
}

Compress_pack(){
    tar zvcfP /var/log/monitor/collect_pack_${CURRENTTIME}.tar.gz ${COLLECT_PATH}
}

interface(){
cat <<EOF
************************************************************
*                                                          *
*              Options                                     *
*                                                          *
************************************************************
           1)ALL
           2)Lw Configuration
           3)Networks Information
           4)Lw Information
           5)System Information
           6)Process Information
           7)Iptables Information
           8)Log
           9)Quit
************************************************************
EOF
}

optionalcollect(){

    for option in ${options[*]};
    do
        case $option in
            1)
                lw_conf_collect
                network_info_collect
                lspci_info_collect
                lw_info_collect
                sys_info_collect
                process_info_collect
                iptables_info_collect
                log_collect
                cpu_usage_first_collect
                interrupts_info_collect
                dev_info_collect
                eth_info_first_collect
                sleep 2
                cpu_usage_second_collect
                interrupts_info_collect
                dev_info_collect
                eth_info_second_collect
                ;;
            2)
                lw_conf_collect
                ;;
            3)
                network_info_collect
                ;;
            4)
                lw_info_collect
                ;;
            5)
                sys_info_collect
                ;;
            6)
                process_info_collect
                ;;
            7)
                iptables_info_collect
                ;;
            8)
                log_collect
                ;;
            9)
                exit
                ;;
            *)
                echo "Invalid option, quit"
                ;;
            esac
    done
    Compress_pack
}

if [ -d /etc/cpeagent ]; then
    CPE_FLAG=1
    agent="cpeagent"
    agent_path=/etc/cpeagent
    agent_log="cpeagent.log"
elif [ -d /etc/popagent ]; then
    POP_FLAG=1
    agent="popagent"
    agent_path=/etc/popagent
    agent_log="popagent.log"
fi

declare -a options
if [ $# -eq 0 ];
then
    interface
    read -p "Please enter your choice:" options
    optionalcollect
else
    options=$*
    optionalcollect
fi
