#!/bin/bash
#v3.1
#2021.01.21
#path
COLLECT_PATH=/var/log/monitor/cpu_monitor
script_name=$(basename $0)
#engine_pcap indicate file
engine_pcap_indicater_file=/tmp/engine_pcap_indicater
#flags
start_flag=0
stop_flag=0
enable_top=0
top_record="/bin/appex/monitor/top_record.sh"
#interval_time
interval_time=10
iteration=0
#percent to trigger
cpu_trigger_percent=85
#max perf files
max_file_num=100
max_pcap_num=300
log_check_days=6
g_last_check_time=0

#cpu usage
declare -a cpu_previous_total_time
declare -a cpu_now_total_time

declare -a cpu_previous_idle_time
declare -a cpu_now_idle_time

declare -a cpu_previous_user_time
declare -a cpu_now_user_time

declare -a cpu_previous_nice_time
declare -a cpu_now_nice_time

declare -a cpu_previous_system_time
declare -a cpu_now_system_time

declare -a cpu_previous_iowait_time
declare -a cpu_now_iowait_time

declare -a cpu_previous_irq_time
declare -a cpu_now_irq_time

declare -a cpu_previous_softirq_time
declare -a cpu_now_softirq_time

declare -a last_trigger_time

pcap_enable=0

help() {
    echo "Usage: $0 <Start/Stop> [SetInterval  <Time>] [SetPercent <percent>] [SetMaxFile <NUM>] [EnablePcap] [EnableTop]"
    echo "      e.g. $0 Start SetInterval 10 SetPercent 85 SetMaxFile 100 EnablePcap EnableTop &"
    echo "NOTE: Interval in unit of second. Default time is 10 second if no interval is specified."
    echo "      SetPercent: Percent to trigger. Default percent is 85"
    echo "      SetMaxFile: Max number of perf files. Default is 100"
    echo "      EnablePcap: Enable engine pcap, default is disable"
    echo "      EnableTop: Enable top info collect,default is disable. make sure /bin/appex/monitor/top_record.sh exist"
}

stop_script(){    
    for pid in `ps aux | grep "${script_name} Start" | grep -v grep | awk '{print $2}'`
    do
        kill -9 $pid > /dev/null 2>&1
    done
    
    if [ ${enable_top} -eq 1 ];then
        ps aux | grep "top_record.sh" | awk '{print $2}' | xargs kill -9 > /dev/null
    fi
    for appex in `ls /proc/net/appex* -d`
    do
        echo 0 > $appex/pcapEnable
    done
}

#cpu_usage_get_time $1=total $2=user $3=nice $4=system $5=idle $6=iowait $7=irq $8=softirq
cpu_usage_get_time(){
    unset $1
    unset $2
    unset $3
    unset $4
    unset $5
    unset $6
    unset $7
    unset $8
    for((i=0;i<${cpu_total_num};i++));do
        stat=`cat /proc/stat | grep "cpu$i" | awk '{print $2" "$3" "$4" "$5" "$6" "$7" "$8}'`
        eval $1[$i]=`echo ${stat} | awk '{printf "%.f",$1+$2+$3+$4+$5+$6+$7}'`
        eval $2[$i]=`echo ${stat} | awk '{print $1}'`
        eval $3[$i]=`echo ${stat} | awk '{print $2}'`
        eval $4[$i]=`echo ${stat} | awk '{print $3}'`
        eval $5[$i]=`echo ${stat} | awk '{print $4}'`
        eval $6[$i]=`echo ${stat} | awk '{print $5}'`
        eval $7[$i]=`echo ${stat} | awk '{print $6}'`
        eval $8[$i]=`echo ${stat} | awk '{print $7}'`
    done
}

pcap_engine(){
    touch $engine_pcap_indicater_file
    for j in $(seq 0 ${engine_factor});do
        engine_num=`expr ${1} + ${j} \* ${cpu_total_num}`
        if [ $engine_num -lt $engine_total_num ];then
            if [ $engine_num -eq 0 ];then
                echo 1 >  /proc/net/appex/pcapEnable
                sleep 10 && echo 0 > /proc/net/appex/pcapEnable && cp -r /appex/log/$engine_num ${COLLECT_PATH}/pcap_engine${engine_num}_$(date +%Y%m%d_%H%M%S) &
            else
                echo 1 >  /proc/net/appex${engine_num}/pcapEnable
                sleep 10 && echo 0 > /proc/net/appex${engine_num}/pcapEnable && cp -r /appex/log/$engine_num ${COLLECT_PATH}/pcap_engine${engine_num}_$(date +%Y%m%d_%H%M%S) &
            fi
            sleep 10 && rm -f $engine_pcap_indicater_file &
        fi
    done
}

cpu_usage_check(){
    time_now=`date +%s`
    cpu_usage_get_time cpu_now_total_time cpu_now_user_time cpu_now_nice_time cpu_now_system_time cpu_now_idle_time cpu_now_iowait_time cpu_now_irq_time cpu_now_softirq_time
    
    for((i=0;i<${cpu_total_num};i++));do
        cpu_num=$i
        cpu_total_time_D_value=`expr ${cpu_now_total_time[i]} - ${cpu_previous_total_time[i]}`
        cpu_idle_time_D_value=`expr ${cpu_now_idle_time[i]} - ${cpu_previous_idle_time[i]}`
        #get cpu usage percentage 
        cpu_usage_percent=$(echo "scale=2; (1 - ${cpu_idle_time_D_value} / ${cpu_total_time_D_value}) * 100" | bc) > /dev/null

        if [ `echo "${cpu_usage_percent} > ${cpu_trigger_percent}" | bc` -eq 1 ];then
            cpu_user_time_D_value=`expr ${cpu_now_user_time[i]} - ${cpu_previous_user_time[i]}`
            cpu_nice_time_D_value=`expr ${cpu_now_nice_time[i]} - ${cpu_previous_nice_time[i]}`
            cpu_system_time_D_value=`expr ${cpu_now_system_time[i]} - ${cpu_previous_system_time[i]}`
            cpu_iowait_time_D_value=`expr ${cpu_now_iowait_time[i]} - ${cpu_previous_iowait_time[i]}`
            cpu_irq_time_D_value=`expr ${cpu_now_irq_time[i]} - ${cpu_previous_irq_time[i]}`
            cpu_softirq_time_D_value=`expr ${cpu_now_softirq_time[i]} - ${cpu_previous_softirq_time[i]}`
            cpu_user_percent=$(echo "scale=2; (${cpu_user_time_D_value} / ${cpu_total_time_D_value}) * 100" | bc) > /dev/null
            cpu_nice_percent=$(echo "scale=2; (${cpu_nice_time_D_value} / ${cpu_total_time_D_value}) * 100" | bc) > /dev/null
            cpu_system_percent=$(echo "scale=2; (${cpu_system_time_D_value} / ${cpu_total_time_D_value}) * 100" | bc) > /dev/null
            cpu_iowait_percent=$(echo "scale=2; (${cpu_iowait_time_D_value} / ${cpu_total_time_D_value}) * 100" | bc) > /dev/null
            cpu_irq_percent=$(echo "scale=2; (${cpu_irq_time_D_value} / ${cpu_total_time_D_value}) * 100" | bc) > /dev/null
            cpu_softirq_percent=$(echo "scale=2; (${cpu_softirq_time_D_value} / ${cpu_total_time_D_value}) * 100" | bc) > /dev/null
            echo "${CURRENT_TIME} cpu$cpu_num usage[${cpu_usage_percent}%] user[${cpu_user_percent}%] nice[${cpu_nice_percent}%] system[${cpu_system_percent}%] iowait[${cpu_iowait_percent}%] irq[${cpu_irq_percent}%] softirq[${cpu_softirq_percent}%]" >>$MONITOR_LOG_FILE
            if [ $time_now -gt ${last_trigger_time[i]} ];then
                #update last trigger time 
                last_trigger_time[i]=`expr $time_now + 1800`
                #perf record
                perf record -C $cpu_num -e cpu-clock -o ${COLLECT_PATH}/perf.record_cpu${cpu_num}_${CURRENT_TIME} -g sleep 10 &
                if [ $pcap_enable -eq 1 ];then
                #enable engine pcap && collect pcap
                    pcap_engine $cpu_num
                fi
            fi
        fi
            
        cpu_previous_total_time[i]=${cpu_now_total_time[i]}
        cpu_previous_user_time[i]=${cpu_now_user_time[i]}
        cpu_previous_nice_time[i]=${cpu_now_nice_time[i]}
        cpu_previous_system_time[i]=${cpu_now_system_time[i]}
        cpu_previous_idle_time[i]=${cpu_now_idle_time[i]}
        cpu_previous_iowait_time[i]=${cpu_now_iowait_time[i]}
        cpu_previous_irq_time[i]=${cpu_now_irq_time[i]}
        cpu_previous_softirq_time[i]=${cpu_now_softirq_time[i]}
    done
}

[[ -z $1 || $1 == "-h" || $1 == "--help" ]] && {
    help
    exit 1
}

until [ $# -eq 0 ]
do
    if [ $1 == "Start" ];then
        start_flag=1
    fi
    if [ $1 == "Stop" ];then
        stop_flag=1
    fi
    if [ $1 == "SetInterval" ];then
        shift
        interval_time=$1
    fi
    if [ $1 == "SetPercent" ];then
        shift
        cpu_trigger_percent=$1
    fi
    if [ $1 == "SetMaxFile" ];then
        shift
        max_file_num=$1
        max_pcap_num=`expr $1 \* 3`
    fi
    if [ $1 == "EnableTop" ];then
        enable_top=1
    fi
    if [ $1 == "EnablePcap" ];then
        pcap_enable=1
    fi
    shift
done

if [ $stop_flag -eq 1 ];then
    stop_script
    exit 1
fi

if [ $start_flag -ne 1 ];then
    exit
fi

is_exist=`ps aux | grep "$script_name Start"| grep -v grep | wc -l` > /dev/null
if [ $is_exist -gt 2 ];then
    echo "$0 already Running"
    exit 1
fi

if [ ${enable_top} -eq 1 ];then
    if [ -f ${top_record} ];then
        setsid ${top_record} & > /dev/null
    else
        echo "${top_record} does not exist, enable failed"
    fi
fi

while true;
do
#time
    CURRENT_TIME=`date +'%y—%m-%d_%H:%M:%S'`
    CURRENT_DATE=`date +'%m_%d'`
#log file
    MONITOR_LOG_FILE=$COLLECT_PATH/cpu_usage_${CURRENT_DATE}.log

#pre-set for first iteration
    if [ $iteration -eq 0 ];then
        #create log path
        if [ ! -d $COLLECT_PATH ]; then
            mkdir -p $COLLECT_PATH
        fi
        #get total cpu numbers 
        cpu_total_num=`cat /proc/stat | grep cpu[0-99] -c`
        #get cpu total/idle time for first iteration
        cpu_usage_get_time cpu_previous_total_time cpu_previous_user_time cpu_previous_nice_time cpu_previous_system_time cpu_previous_idle_time cpu_previous_iowait_time cpu_previous_irq_time cpu_previous_softirq_time
        #pre-set for last triggered time
        for i in $(seq 0 ${cpu_total_num});do
            last_trigger_time[i]=`date +%s`
        done
        #get engine numbers & get engine factor to multiply with for finding corresponding engine
        if [ -d /proc/net/appex ]; then
            engine_total_num=`ls /proc/net/appex* -d | wc -l`
            engine_factor=`expr $engine_total_num / $cpu_total_num`
            if [ $(( $engine_total_num % $cpu_total_num )) -ne 0 ];then
                engine_factor=`expr ${engine_factor} + 1`
            fi
            if [ $engine_total_num -lt $cpu_total_num ];then
                engine_factor=0
            fi
        else
            pcap_enable=0
        fi
        iteration=`expr $iteration + 1`
    fi
    
#log delete 
    perf_file_num=`find ${COLLECT_PATH} -name "perf.record_cpu*" | wc -l`
    if [ $perf_file_num -gt $max_file_num ];then
        del_file_num=`expr $perf_file_num - $max_file_num`
        find ${COLLECT_PATH} -type f -name "perf.record_cpu*" | xargs ls -ta | tail -n ${del_file_num} | xargs rm -rf
    fi
    
    pcap_file_num=`find ${COLLECT_PATH} -name "pcap_engine*" | wc -l`
    if [ $pcap_file_num -gt $max_pcap_num ];then
        pcap_del_file_num=`expr $pcap_file_num - $max_pcap_num`
        find ${COLLECT_PATH} -type d -name "pcap_engine*" | xargs ls -dta | tail -n ${pcap_del_file_num} | xargs rm -rf
    fi

    time_now=`date +%s`
    if [ ${time_now} -gt ${g_last_check_time} ];then
        #check every 12hrs
        g_last_check_time=`expr $time_now + 43200`
        find ${COLLECT_PATH} -maxdepth 1 -mtime +${log_check_days} -type f -name "cpu_usage_*" -execdir rm -rf {} \;
        find ${COLLECT_PATH} -maxdepth 1 -mtime +${log_check_days} -type f -name "perf.record_cpu*" -execdir rm -rf {} \;
        find ${COLLECT_PATH} -maxdepth 1 -mtime +${log_check_days} -type d -name "pcap_engine*" -execdir rm -rf {} \;
    fi
    
    sleep ${interval_time}
    cpu_usage_check
done;
