#!/bin/bash
#v1.6
#2021.01.07
HL_START="\033[37;40;1m"
HL_END="\033[0m" 
declare -a queue_name           #Net device queue name e.g eth2-TxRx-0
declare -a queue_id             #Interrupts id corresponding to net device queue e.g eth2-TxRx-0 :corresponding interrupts id 50
declare -a net_interface        #Physical net interface name e.g eth2
declare -a net_device_driver    #Driver name e.g ixgbe
delegate_cpu_num_igb=0
match_count=0

#drivers supports net device queue
match_driver=(
    "i40e"
    "ixgbe"
    "igb"       #note: For igb driver, max queue num should be found by ethtool -l, usually it's less than 4, so cpu it's delegate to should be counted seperately in order to well-delegated
)

#delegate_cpu $1:net_interface $2: optional "is_igb" indicate that net device driver for corresponding net device($1) is igb
#This function find out corresponding net device queue interrupt ids for net_interface($1) and delegate them to round-robin count cpu
delegate_cpu(){
    unset queue_name
    unset queue_id
    
    eth_name=$1
    if [ "$2" == "is_igb" ];then
        delegate_cpu_num=${delegate_cpu_num_igb}
    else
        delegate_cpu_num=0
    fi
    queue_names=`cat /proc/interrupts | grep ${eth_name} | awk '{print $'$queue_name_row'}'` > /dev/null
    queue_ids=`cat /proc/interrupts | grep ${eth_name} | awk '{print $1}' | sed 's/://g'` > /dev/null
    queen_lines=`cat /proc/interrupts | grep ${eth_name} | wc -l` > /dev/null
    for((i=1;i<=${queen_lines};i++));do
        eval queue_name[i]=`echo ${queue_names} | awk '{print $'$i'}'`  > /dev/null
        eval queue_id[i]=`echo ${queue_ids} | awk '{print $'$i'}'`  > /dev/null
    done 

    for((i=1;i<=${#queue_id[@]};i++));do
        if [ ${queue_name[i]} == ${eth_name} ];then
            continue
        fi
        if [ $[delegate_cpu_num] -eq ${cpu_num} ];then
            delegate_cpu_num=0
        fi
        echo ${delegate_cpu_num} > /proc/irq/${queue_id[i]}/smp_affinity_list 
        cmd="$cmd /usr/bin/printf \"Net Device queue:$HL_START%s$HL_END\" ${queue_name[i]};"
        cmd="$cmd /usr/bin/printf \" Interrupt Id:$HL_START%d$HL_END\" ${queue_id[i]};"
        cmd="$cmd /usr/bin/printf \" Delegated to Cpu:$HL_START%d$HL_END\" ${delegate_cpu_num};"
        cmd="$cmd /usr/bin/printf \"\n\";"  
        let delegate_cpu_num=${delegate_cpu_num}+1
    done
    if [ "$2" == "is_igb" ];then
        delegate_cpu_num_igb=${delegate_cpu_num}
    fi
}

cpu_num=`cat /proc/cpuinfo |grep "processor"|sort -u|wc -l` > /dev/null
if [ ${cpu_num} -eq 1 ];then
    exit
fi
queue_name_row=`expr ${cpu_num} + 3` > /dev/null

#stop irqbalance
exist=`ps aux | grep irqbalance | grep -v grep` > /dev/null
if [ -n "${exist}" ];then
    systemctl stop irqbalance > /dev/null
fi

#find out Physical net interface and their driver
for device in `ls /sys/class/net/`;do

    if [ ! -d /sys/class/net/${device}/device ];then
        continue
    fi

    driver=`ethtool -i $device | grep driver | awk '{print $2}'`
    for((i=0;i<${#match_driver[@]};i++));do
        if [ ${driver} == "${match_driver[i]}" ];then
            net_interface[${match_count}]="${device}"
            net_device_driver[${match_count}]=${driver}
            match_count=`expr ${match_count} + 1`
            break
        fi
    done
done

#set device channels 
for((k=0;k<${#net_interface[@]};k++));do
    if [ ${net_device_driver[k]} == "igb" ];then
        igb_queue_max=`ethtool -l ${net_interface[k]} | grep Combined | sed -n 1p | awk '{print $2}'` > /dev/null 2>&1
        if [ ${cpu_num} -lt ${igb_queue_max} ];then
            ethtool -L ${net_interface[k]} combined ${cpu_num} > /dev/null 2>&1
        else
            ethtool -L ${net_interface[k]} combined ${igb_queue_max} > /dev/null 2>&1
        fi
        delegate_cpu ${net_interface[k]} "is_igb"
    else
        ethtool -L ${net_interface[k]} combined ${cpu_num} > /dev/null 2>&1
        delegate_cpu ${net_interface[k]}
    fi
    echo "Handling ${net_interface[k]}"
done

eval $cmd


