#!/bin/sh
#v1.0.6
#2020.08.22
#gloabal variable, edit carefully
gVersion=1.0.6
#default check interval, second
gInterval=60

#default check timeout, second
gTimeout=3
script_name=$0

is_exist=`ps aux | grep $script_name | grep -v grep | wc -l` > /dev/null
if [ $is_exist -gt 2 ];then
    echo "$0 already Running"
    exit 1
fi

if [ $# -ne 0 ];then
    if [ $1 == "EnableDelay" ];then
        sleep 120
    fi
fi

if [ -z `cat /proc/net/lw-conf/lan-tbl | grep Statistics` ];then
    wanGateWay=`cat /proc/net/lw-conf/wan-tbl | awk 'NR>1{print $5}' | grep -v "0.0.0.0"`
    lanGateWay=`cat /proc/net/lw-conf/lan-tbl | awk 'NR>1{print $4}' | grep -v "0.0.0.0"`
else
    wanGateWay=`cat /proc/net/lw-conf/wan-tbl | grep GatewayIp | grep -v "0.0.0.0" | awk -F "GatewayIp: " '{print $2}'`
    lanGateWay=`cat /proc/net/lw-conf/lan-tbl |grep NextHop | grep -v "0.0.0.0" | awk -F "NextHop: " '{print $2}'`
fi

#ping
gPingTarget=(
$wanGateWay
$lanGateWay
)
gPingTimeout=$gTimeout
gPingCount=6

#tcping
gTcpingTarget=(

)
gTcpingTimeout=$gTimeout


#wget
gWgetTarget=(

)
gWgetTimeout=$gTimeout

#log
gPingLogPath=/var/log/monitor/detect_ping/
gTcpingLogPath=/var/log/monitor/detect_tcping/
gWgetLogPath=/var/log/monitor/detect_wget/

#log check days
gLogCheckDay=7

#log check interval
gLogCheckInterval=60

checkLogFile()
{
    find $gPingLogPath -type f -mtime +$gLogCheckDay -execdir rm -- '{}' \; 
    find $gTcpingLogPath -type f -mtime +$gLogCheckDay -execdir rm -- '{}' \; 
    find $gWgetLogPath -type f -mtime +$gLogCheckDay -execdir rm -- '{}' \; 
}

pinglog()
{
    [ -f "$gPingLogFile" ] && echo "$(date +'%Y%m%d-%H:%M:%S') $@" >> $gPingLogFile
}
pingnotimelog()
{
    [ -f "$gPingLogFile" ] && echo "$@" >> $gPingLogFile
}

tcpinglog()
{
    [ -f "$gTcpingLogFile" ] && echo "$(date +'%Y%m%d-%H:%M:%S') $@" >> $gTcpingLogFile
}
tcpingnotimelog()
{
    [ -f "$gTcpingLogFile" ] && echo "$@" >> $gTcpingLogFile
}

wgetlog()
{
    [ -f "$gWgetLogFile" ] && echo "$(date +'%Y%m%d-%H:%M:%S') $@" >> $gWgetLogFile
}
wgetnotimelog()
{
    [ -f "$gWgetLogFile" ] && echo "$@" >> $gWgetLogFile
}

initConfig()
{
    [ -z "$gInterval" -o $gInterval -eq 0  ] && {
        gInterval=60
    }
    
    [ -z "$gTimeout" -o $gTimeout -eq 0  ] && {
        gTimeout=3
    }

    [ -z "$gPingCount" -o $gPingCount -eq 0  ] && {
        gPingCount=6
    }

    [ -z "$gPingTimeout" -o $gPingTimeout -eq 0  ] && {
        gPingTimeout=$gTimeout
    }

    [ -z "$gTcpingTimeout" -o $gTcpingTimeout -eq 0  ] && {
        gTcpingTimeout=$gTimeout
    }

    [ -z "$gWgetTimeout" -o $gWgetTimeout -eq 0  ] && {
        gWgetTimeout=$gTimeout
    }
    
    if [ ${gLogCheckDay} -lt 1 ];then
        gLogCheckDay=0
    else
        gLogCheckDay=`expr ${gLogCheckDay} - 1`
    fi

    [ -z "$gPingLogPath" ] && {
        gPingLogPath=/var/log/monitor/detect_ping/
    }
    mkdir -p $gPingLogPath 2>/dev/null

    [ -z "$gTcpingLogPath" ] && {
        gTcpingLogPath=/var/log/monitor/detect_tcping/
    }
    mkdir -p $gTcpingLogPath 2>/dev/null

    [ -z "$gWgetLogPath" ] && {
        gWgetLogPath=/var/log/monitor/detect_wget/
    }
    mkdir -p $gWgetLogPath 2>/dev/null
    updateLogFileName
}

updateLogFileName()
{
    local logPrefix

    logPrefix=$(date +'%Y%m%d')

    gPingLogFileName=ping-${logPrefix}.log
    gPingLogFile=$gPingLogPath/$gPingLogFileName
    [ -f "$gPingLogFile" ] || touch $gPingLogFile

    gTcpingLogFileName=tcping-${logPrefix}.log
    gTcpingLogFile=$gTcpingLogPath/$gTcpingLogFileName
    [ -f "$gTcpingLogFile" ] || touch $gTcpingLogFile

    gWgetLogFileName=wget-${logPrefix}.log
    gWgetLogFile=$gWgetLogPath/$gWgetLogFileName
    [ -f "$gWgetLogFile" ] || touch $gWgetLogFile
}

dumpCfg()
{
    pingnotimelog "configuration:"
    pingnotimelog "Ping: count = $gPingCount timeout = $gPingTimeout"
    for target in "${gPingTarget[@]}"
    do
        pingnotimelog "    target: $target"
    done
    pingnotimelog "log: file = $gPingLogFile"
    pingnotimelog ""

    tcpingnotimelog "configuration:"
    tcpingnotimelog "Tcping: timeout = $gTcpingTimeout"
    for target in "${gTcpingTarget[@]}"
    do
        tcpingnotimelog '    target: '"$target"
    done
    if [ ${#gTcpingTarget[@]} == 0 ];then
        tcpingnotimelog '    target: '"No tcping target set"
    fi
    tcpingnotimelog "log: file = $gTcpingLogFile"
    tcpingnotimelog ""

    wgetnotimelog "configuration:"
    wgetnotimelog "Wget: timeout = $gWgetTimeout"
    for target in "${gWgetTarget[@]}"
    do
        wgetnotimelog "    target: $target"
    done
    if [ ${#gWgetTarget[@]} == 0 ];then
        wgetnotimelog '    target: '"No Wget target set"
    fi
    wgetnotimelog "log: file = $gWgetLogFile"
    wgetnotimelog ""

}


pingDetect()
{
    local target=$1 
    local result 

    [ -z "$target" ] && return

    result=$(ping -i 0.3 -q -c $gPingCount -w $gPingTimeout ${target} | grep "packet loss" | awk '{print $6}' | awk -F "%" '{print $1}')
    if [ $result -ne 0 ];then
        pinglog "Ping $target: packet loss ${result}%"
    fi
}
tcpingDetect()
{
    local target=$1
    local port=$2
    local result

    [ -z "$target" ] && return
    [ -z "$port" ] && return 

    result=$(tcping -t $gTcpingTimeout $target $port 2>&1)
    tcpinglog "${result}"
}

wgetgDetect()
{
    local target=$1

    [ -z "$target" ] && return

    result=$(wget --no-check-certificate -t 1 -T $gWgetTimeout $target -O /dev/null 2>&1 |grep -vE '[0-9]s|^$|\.\.\.\.')
    wgetlog "wget $target"
    wgetnotimelog "$result"
    wgetnotimelog ""
}


gProgFile=$(readlink -f $0)
gProg=$(basename $0)

initConfig

pinglog "${gProg}-$gVersion[$$] start running ..."
tcpinglog "${gProg}-$gVersion[$$] start running ..."
wgetlog "${gProg}-$gVersion[$$] start running ..."
dumpCfg

count=0
tcpingnewline=0
wgetnewline=0
while true
do
    updateLogFileName
    [ $count -gt 0 -a $(expr $count % $gLogCheckInterval) -eq 0 ] && {
        checkLogFile
    }

    [ $tcpingnewline -eq 1 ] && {
        tcpingnotimelog ""
    }
    [ $wgetnewline -eq 1 ] && {
        wgetnotimelog ""
        wgetnotimelog ""
    }
    
    for target in "${gPingTarget[@]}"
    do
        pingDetect $target &
    done

    tcpingnewline=0
    for target in "${gTcpingTarget[@]}"
    do
        tcpingnewline=1
        entry=($target)
        tcpingDetect ${entry[0]} ${entry[1]}
    done

    wgetnewline=0
    for target in "${gWgetTarget[@]}"
    do
        wgetnewline=1
        wgetgDetect $target 
    done

    count=$(expr $count + 1)
    sleep $gInterval
done

