#!/bin/bash
#v2.2
#2021.05.11
Monitor_Path=/var/log/monitor/drop_monitor
interface_log_file=${Monitor_Path}/.interface_drop.tmp
conn_log_file=${Monitor_Path}/.conn_drop.tmp
engine_log_file=${Monitor_Path}/.engine_drop.tmp
pid_file=${Monitor_Path}/drop_monitor.pid

script_name=$(basename $0)
webhook=https://oapi.dingtalk.com/robot/send?access_token=499f1b084ce4eee411ce1a482dfb801588658386e7e540a2320a4844810c903a
conf_file=/etc/cpeagent/cpeagent.conf

g_interval=60
start_flag=0
stop_flag=0
alert_on=0

declare -A int_drop_pre
declare -A int_drop_now
declare -A int_drop_diff
declare -A conn_drop_pre
declare -A conn_drop_now
declare -A conn_drop_diff
declare -A engine_drop_pre
declare -A engine_drop_now
declare -A engine_drop_diff

int_msg=""
conn_msg=""
engine_msg=""

help() {
    echo "Usage: $script_name <Start/Stop> [SetInterval  <Time>] [EnableAlert]"
    echo "      e.g. $script_name Start SetInterval 60 EnableAlert &"
    echo "NOTE: Interval in unit of second. Default time is 60 second if no interval is specified."
    echo "      EnableAlert: Enable DingDing alert.(optional)"
}

send_message()
{
    curl -s $webhook -H 'Content-Type: application/json' -d "
    {
        'msgtype': 'text',
        'text': {
            'content': 'Packet Drop Alert:\n${site_name}\n##### DevId :${dev_id}\nDate:`date`\n$1'
        },
        'at': {
            'isAtAll': true
        }
    }" > /dev/null
}

stop_script(){
    for pid in `ps aux | grep "$script_name Start" | grep -v grep | awk '{print $2}'`
    do
        kill -9 $pid
    done
}

interface_drop_collect(){
    interface_drop_now=`cat /proc/net/dev | awk '$5!=0' | awk '{print $1" "$5 }' | sed 's/://g' | grep -v drop | grep -v Inter`
    
    if [ -n "$interface_drop_now" ];then
        echo "=============${CURRENT_TIME}=============" >> ${log_file}
        echo -e "interface_drop_now :\n${interface_drop_now}" >> ${log_file}
    fi
    [ $alert_on -eq 1 ] && {
        echo -e "${interface_drop_now}" > $interface_log_file
    }
}

conn_drop_collect(){
    conn_drop_now=`cat /proc/net/lw-conf/ltt/stats | grep drop | awk '{print $1" "$2}'`

    if [ -n "$conn_drop_now" ];then
        echo "=============${CURRENT_TIME}=============" >> ${log_file}
        echo -e "conn_drop_now :\n${conn_drop_now}" >> ${log_file}
    fi
    [ $alert_on -eq 1 ] && {
        echo -e "${conn_drop_now}" > $conn_log_file
    }
}

single_engine_drop_collect(){
    single_engine_drop_now=`cat /proc/net/$1/stats | grep Discards | grep -v 'ReasDiscards\|SchdAckMergeDiscards'| sed 's/\s*//g' | sed 's/=/ /g' | sed 's/\./_/g'`
    if [ -n "$single_engine_drop_now" ];then
        echo "=============${CURRENT_TIME}=============" >> ${log_file}
        echo -e "For $1:\n${single_engine_drop_now}" >> ${log_file}
    fi
    [ $alert_on -eq 1 ] && {
        echo -e "${single_engine_drop_now}" > ${engine_log_file}_${1}
    }
}

engine_drop_collect(){
    for engine in `ls /proc/net | grep appex`
    do
        single_engine_drop_collect $engine
    done
}

interface_get_diff_msg(){
    int_msg=""
    while read line
    do
        name=`echo $line | awk '{print $1}'`
        val=`echo $line | awk '{print $2}'`
        if [ -n "$name" -a -n "$(echo $val | sed -n "/^[0-9]\+$/p")" ];then
            eval int_drop_now["$name"]="$val"
            
            if [ -n "${int_drop_pre["$name"]}" ];then
                int_drop_diff["$name"]=$((${int_drop_now["$name"]} - ${int_drop_pre["$name"]}))
                if [ ${int_drop_diff["$name"]} -ne 0 ];then
                    int_msg+="$name=${int_drop_diff["$name"]} \n"
                fi
            else
                int_drop_diff["$name"]=0
            fi
        fi
    done < $interface_log_file

    for key in ${!int_drop_now[*]};do
        int_drop_pre["$key"]="${int_drop_now["$key"]}"
    done
}

conn_get_diff_msg(){
    conn_msg=""
    while read line
    do
        name=`echo $line | awk '{print $1}'`
        val=`echo $line | awk '{print $2}'`
        if [ -n "$name" -a -n "$(echo $val | sed -n "/^[0-9]\+$/p")" ];then
            eval conn_drop_now["$name"]="$val"
            if [ -n "${conn_drop_pre["$name"]}" ];then
                conn_drop_diff["$name"]=$((${conn_drop_now["$name"]} - ${conn_drop_pre["$name"]}))
                if [ ${conn_drop_diff["$name"]} -ne 0 ];then
                    conn_msg+="$name=${conn_drop_diff["$name"]} \n"
                fi
            else
                conn_drop_diff["$name"]=0
            fi
        fi
    done < $conn_log_file

    for key in ${!conn_drop_now[*]};do
        conn_drop_pre["$key"]="${conn_drop_now["$key"]}"
    done
}

single_engine_get_diff_msg(){
    while read line
    do
        name=`echo $line | awk '{print $1}'`
        val=`echo $line | awk '{print $2}'`
        if [ -n "$name" -a -n "$(echo $val | sed -n "/^[0-9]\+$/p")" ];then
            eval engine_drop_now["$1_$name"]="$val"
            
            if [ -n "${engine_drop_pre["$1_$name"]}" ];then
                engine_drop_diff["$1_$name"]=$((${engine_drop_now["$1_$name"]} - ${engine_drop_pre["$1_$name"]}))
                if [ ${engine_drop_diff["$1_$name"]} -ne 0 ];then
                    engine_msg+="$1_$name=${engine_drop_diff["$1_$name"]} \n"
                fi
            else
                engine_drop_diff["$1_$name"]=0
            fi
        fi
    done < ${engine_log_file}_${1}

    for key in ${!engine_drop_now[*]};do
        engine_drop_pre["$key"]="${engine_drop_now[$key]}"
    done
}

engine_get_diff_msg(){
    engine_msg=""
    for engine in `ls /proc/net | grep appex`
    do
        single_engine_get_diff_msg $engine
    done
}

send_interface_drop_msg(){
    if [ -n "$int_msg" ];then
        send_message "Interface Drop Alert(D-value)\n$int_msg"
    fi
}

send_conn_drop_msg(){
    if [ -n "$conn_msg" ];then
        send_message "Conn Drop Alert(D-value)\n$conn_msg"
    fi
}

send_engine_drop_msg(){
    if [ -n "$engine_msg" ];then
        send_message "Engine Drop Alert(D-value)\n$engine_msg"
    fi
}

[[ $1 == "-help" || $1 == "--help" ]] && {
    help
    exit 1
}

until [ $# -eq 0 ]
do
    if [ $1 == "Start" ];then
        start_flag=1
    fi
    if [ $1 == "Stop" ];then
        stop_flag=1
    fi
    if [ $1 == "EnableAlert" ];then
        alert_on=1
    fi
    if [ $1 == "SetInterval" ];then
        shift
        g_interval=$1
    fi
    shift
done

if [ $stop_flag -eq 1 ];then
    stop_script
    exit 1
fi

if [ $start_flag -ne 1 ];then
    help
    exit 1
fi

if [ $alert_on -eq 1 ];then
    if [ -d /etc/popagent ]; then
        conf_file=/etc/popagent/popagent.conf
    fi

    dev_id=`cat ${conf_file} | grep 'popid\|cpeid' | cut -d \= -f 2`
    site_name=`cat /etc/motd | sed '/^$/d'`
    [ ! -n "${site_name}" -o ! -n "${dev_id}" ] && {
        echo "invalid parameter or ${conf_file} missing"
        exit 1
    }
fi

if [ ! -d ${Monitor_Path} ];then
    mkdir -p $Monitor_Path
fi

if [ ! -f $pid_file ]; then 
    touch $pid_file
fi

pid=`cat $pid_file`
ps aux | awk '{print $2}'| grep "^$pid$"
if [ $? -eq 0 ]; then
    echo "$script_name already Running"
    exit 1
fi
echo $$ > $pid_file

while true
do
    CURRENT_TIME=`date +'%y—%m-%d_%H:%M:%S'`
    log_file=${Monitor_Path}/drop_monitor_`date "+%Y-%m-%d"`.log

    interface_drop_collect
    conn_drop_collect
    engine_drop_collect
    
    if [ $alert_on -eq 1 ];then
        interface_get_diff_msg
        conn_get_diff_msg
        engine_get_diff_msg
        
        send_interface_drop_msg
        send_conn_drop_msg
        send_engine_drop_msg
    fi

    find ${Monitor_Path} -type f -name "drop_monitor_*" -mtime +7 -execdir rm -- '{}' \;
    sleep $g_interval
done
