#!/bin/bash
#/bin/appex/collect/lw_collect_rt.sh
#v3.00 2021-02-04

PATH=$PATH:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/bin/appex/:/root/bin
LOGPATH=/var/log/monitor/collect_rt
LOGFILE=${LOGPATH}/collect_rt.log
ROTATEFILE=/etc/logrotate.d/collect_rt_log
PIDFILE=${LOGPATH}/lw_collect_rt.pid

if [ ! -d ${LOGPATH} ]; then
    mkdir -p ${LOGPATH}
fi

if [ ! -f $PIDFILE ]; then 
    touch $PIDFILE
fi

PID=`cat $PIDFILE`
ps aux|awk '{print $2}'|grep "^$PID$"
if [ $? -eq 0 ]; then
    exit
fi
echo $$ > $PIDFILE

echo "$LOGFILE {"               >  $ROTATEFILE
echo "    hourly"               >> $ROTATEFILE
echo "    size 100M"            >> $ROTATEFILE
echo "    copytruncate"         >> $ROTATEFILE
echo "    rotate 50"            >> $ROTATEFILE
echo "    compress"             >> $ROTATEFILE
echo "    dateext"              >> $ROTATEFILE
echo "    dateformat  -%Y%m%d%s">> $ROTATEFILE
echo "    notifempty"           >> $ROTATEFILE
echo "    maxage 7"             >> $ROTATEFILE
echo "}"                        >> $ROTATEFILE

if [ -d /proc/ltt ];then
    LTTPATH=/proc/ltt
elif [ -d /proc/net/lw-conf/ltt ];then
    LTTPATH=/proc/net/lw-conf/ltt
fi

while true; do
    echo ============================== >> $LOGFILE
    date >> $LOGFILE
    # stats
    echo ---------LTT----------------- >> $LOGFILE
    cat $LTTPATH/stats >> $LOGFILE
    echo >> $LOGFILE
    cat $LTTPATH/lsw-tbl >> $LOGFILE
    echo >> $LOGFILE
    cat $LTTPATH/tunnels >> $LOGFILE
    echo >> $LOGFILE
    cat $LTTPATH/link-tbl >> $LOGFILE
    echo >> $LOGFILE
    cat $LTTPATH/cpe-iptbl >> $LOGFILE
    echo >> $LOGFILE
    cat $LTTPATH/pop-iptbl >> $LOGFILE
    echo >> $LOGFILE

    #ifconfig
    echo -----------ifconfig-------------- >> $LOGFILE
    ifconfig >> $LOGFILE

    #interrupt
    echo -----------/proc/interrupts----------- >> $LOGFILE
    grep eth /proc/interrupts >> $LOGFILE

    # qdisc
    echo ---------------qdisc----------- >> $LOGFILE
    tc -s qdisc show >> $LOGFILE

    #memory
    echo ---------------memory----------- >> $LOGFILE
    free >> $LOGFILE
    
    #cpustats
    echo ---------------cpu stats----------- >> $LOGFILE
    mpstat -P ALL >> $LOGFILE

    #disk usage
    echo ---------------disk usage----------- >> $LOGFILE
    df -h >> $LOGFILE
    
    #route
    echo ---------------route----------- >> $LOGFILE
    route -n >> $LOGFILE

    #arp
    echo ---------------arp----------- >> $LOGFILE
    arp -n >> $LOGFILE   

    #engine stats
    appexlist=`ls /proc/net/appex* -d`
    count=0
    for DR in $appexlist
    do
                let "count++"
    done
    num=`expr $count - 1`

    for i in $(seq 0 ${num});
    do
        echo ------------appex$i stats-------- >> $LOGFILE
        if [ $i -eq 0 ];then
            echo displayLevel: 10 > /proc/net/appex/cmd
            cat /proc/net/appex/stats >> $LOGFILE
        else
            echo displayLevel: 10 > /proc/net/appex$i/cmd
            cat /proc/net/appex$i/stats >> $LOGFILE
        fi
    done

    #sar 
    echo -----------sar report---------- >> $LOGFILE
    sar -n DEV,EDEV 1 1 >> $LOGFILE

    sleep 60
done

