#!/bin/bash
#v1.5
#2021-01-01
#path
COLLECT_PATH=/var/log/monitor/mem_collect
script_name=$(basename $0)
#flags
start_flag=0
stop_flag=0
#interval_time
interval_time=10
mem_usage_max_percent=85
max_file_num=100

help() {
    echo "Usage: $0 <Start/Stop> [SetInterval  <Time>] [SetPercent <percent>] [SetMaxFile <NUM>]"
    echo "      e.g. $0 Start SetInterval 20 SetPercent 85 SetMaxFile 100 &"
    echo "NOTE: Interval in unit of minute. Default time is 10 minutes if no interval is specified."
    echo "      Percent to collect info. Default percent is 85"
    echo "      Max number of collect files. Default is 100"
}
stop_script(){
    for pid in `ps aux | grep "${script_name} Start" | grep -v grep | awk '{print $2}'`
    do
        kill -9 $pid > /dev/null 2>&1
    done
}
check_mem_usage(){
    mem_stat=`free -m | grep Mem`
    mem_usaged=`echo ${mem_stat} | awk '{print $3}'` > /dev/null
    mem_total=`echo ${mem_stat} | awk '{print $2}'` > /dev/null
    mem_cached=`echo ${mem_stat} | awk '{print $6}'` > /dev/null
    mem_usaged_total=`expr ${mem_usaged} + ${mem_cached}`
    mem_usage_percent=$(echo "scale=2; ${mem_usaged_total} / ${mem_total} * 100" | bc) > /dev/null
    if [ `echo "${mem_usage_percent} > ${mem_usage_max_percent}" | bc` -eq 1 ];then
        echo "=======================$CURRENT_TIME=======================" >> $MONITOR_LOG_FILE
        echo "=======================free=======================" >> $MONITOR_LOG_FILE
        free >> $MONITOR_LOG_FILE
        echo "=======================du -sh /tmp/* =======================" >> $MONITOR_LOG_FILE
        du -sh /tmp/* >> $MONITOR_LOG_FILE
        echo "=======================ps axu=======================" >> $MONITOR_LOG_FILE
        ps axu >> $MONITOR_LOG_FILE
        echo "=======================cpeagent mem=======================" >> $MONITOR_LOG_FILE
        cpeagent mem >> $MONITOR_LOG_FILE
        echo "=======================/proc/meminfo=======================" >> $MONITOR_LOG_FILE
        cat /proc/meminfo >> $MONITOR_LOG_FILE
        echo "=======================/proc/slabinfo=======================" >> $MONITOR_LOG_FILE
        cat /proc/slabinfo >> $MONITOR_LOG_FILE
        echo "=======================/proc/vmallocinfo=======================" >> $MONITOR_LOG_FILE
        cat /proc/vmallocinfo >> $MONITOR_LOG_FILE
        echo "=======================/proc/net/lw-conf/memusage=======================" >> $MONITOR_LOG_FILE
        cat /proc/net/lw-conf/memusage >> $MONITOR_LOG_FILE
        echo "=======================/proc/net/lw-conf/mempoolusage=======================" >> $MONITOR_LOG_FILE
        cat /proc/net/lw-conf/mempoolusage >> $MONITOR_LOG_FILE
        echo "=======================ss -a=======================" >> $MONITOR_LOG_FILE
        ss -a >> $MONITOR_LOG_FILE
        echo "=======================lsmod | grep balloon=======================" >> $MONITOR_LOG_FILE
        lsmod | grep balloon >> $MONITOR_LOG_FILE
        echo "=======================/proc/net/appex*/stats=======================" >> $MONITOR_LOG_FILE
        cat /proc/net/appex*/stats >> $MONITOR_LOG_FILE
    fi  
}
[[ -z $1 || $1 == "-help" || $1 == "--help" ]] && {
    help
    exit 1
}
until [ $# -eq 0 ]
do
    if [ $1 == "Start" ];then
        start_flag=1
    fi
    if [ $1 == "Stop" ];then
        stop_flag=1
    fi
    if [ $1 == "SetInterval" ];then
        shift
        interval_time=$1
    fi
    if [ $1 == "SetPercent" ];then
        shift
        mem_usage_max_percent=$1
    fi
	if [ $1 == "SetMaxFile" ];then
        shift
        max_file_num=$1
    fi
    shift
done

if [ $stop_flag -eq 1 ];then
    stop_script
    exit 1
fi

if [ $start_flag -ne 1 ];then
    exit
fi

is_exist=`ps aux | grep "${script_name} Start"| grep -v grep | wc -l` > /dev/null
if [ $is_exist -gt 2 ];then
    echo "$0 already Running"
    exit 1
fi

while true;
do
#time
    CURRENT_TIME=`date +'%y—%m-%d_%H_%M_%S'`
#file
    if [ ! -d $COLLECT_PATH ]; then
        mkdir -p $COLLECT_PATH
    fi
    MONITOR_LOG_FILE=${COLLECT_PATH}/mem_collect_${CURRENT_TIME}.log

    check_mem_usage
    collect_file_num=`find ${COLLECT_PATH} -type f -name "mem_collect_*" | wc -l`
    if [ $collect_file_num -gt $max_file_num ];then
        del_file_num=`expr $collect_file_num - $max_file_num`
        find ${COLLECT_PATH} -type f -name "mem_collect_*" | xargs ls -ta | tail -n ${del_file_num} | xargs rm -rf
    fi
    sleep ${interval_time}m
done;
