#!/bin/bash
#2020-4-3
#Ziwei Xu

#path
MOMITOR_PATH=/var/log/monitor
#flags
start_flag=0
stop_flag=0
vmalloc_flag=0
slab_flag=0
#interval_time
interval_time=1
cycles=0
#percent to clean cache
mem_usage_max_percent=85
#monitored parameters
#meminfo
declare -a meminfo_parameters
declare -a meminfo_previous
declare -a meminfo_now
declare -a meminfo_D_value
#mempool
declare -a mempool_parameters
declare -a mempool_blocksize_previous
declare -a mempool_blocknum_previous
declare -a mempool_totalmem_previous
declare -a mempool_blocksize_now
declare -a mempool_blocknum_now
declare -a mempool_totalmem_now
declare -a mempool_D_value
#memusage
declare -a memusage_parameters
declare -a memusage_previous
declare -a memusage_counter_previous
declare -a memusage_now
declare -a memusage_counter_now
declare -a memusage_D_value
#vmallocinfo
declare -a vmalloc_parameters
declare -a vmalloc_previous
declare -a vmalloc_now
declare -a vmalloc_D_value
declare -a vmalloc_multiple_parameters
multi_count=0
parameter_exist=0
#slabinfo
declare -a slab_parameters_previous
declare -a slab_objsize_previous
declare -a slab_objnum_previous
declare -a slab_total_previous
declare -a slab_parameters_now
declare -a slab_objsize_now
declare -a slab_objnum_now
declare -a slab_total_now
declare -a slab_D_value
declare -a slab_parameters_tmp
declare -a slab_total_tmp
#threshhold to alert
threshhold=100

help() {
	echo "Usage: $0 <Start/Stop> [EnableVmalloc] [EnableSlab] [SetInterval  <Time>] [SetThreshhold  <kB>] [SetPercent <percent>]"
	echo "      e.g. $0 Start EnableVmalloc EnableSlab SetInterval 20 SetThreshhold 100 SetPercent 85 &"
    echo "NOTE: Interval in unit of minute. Default time is 10 minutes if no interval is specified."
    echo "      Threshhold to alert in unit of kB. Default threshhold is 100 kB"
    echo "      Percent to cleann cache. Default percent is 85"
    echo "      EnableVmalloc/EnableSlab to collect vmallocinfo and slabinfo."
}

stop_script(){
    declare -a script_pid
    script_pid=`ps aux | grep "mem_monitor.sh Start"| grep -v grep | awk '{print $2}'` > /dev/null
    
    for pid in $script_pid; do
        if [ $pid ]; then
            kill -9 $pid > /dev/null 2>&1
        fi
    done
}

meminfo_report(){
    if [ $cycles -eq 0 ];then
        for i in $(seq 1 ${mem_info_lines});do
            meminfo_parameters[i]=`cat /proc/meminfo | awk '{print $1}'|sed 's/://g' | sed -n ${i}p`> /dev/null
            meminfo_previous[i]=`cat /proc/meminfo | awk '{print $2}'| sed -n ${i}p` > /dev/null 
        done
    else
        echo ======================================================================================== >> $REPORT_LOG_FILE
        echo -------------meminfo_report-------------- >> $REPORT_LOG_FILE
        for i in $(seq 1 ${mem_info_lines});do
            meminfo_now[i]=`cat /proc/meminfo | awk '{print $2}'| sed -n ${i}p` > /dev/null
            meminfo_D_value[i]=`expr ${meminfo_now[i]} - ${meminfo_previous[i]}`> /dev/null
            if [ ${meminfo_D_value[i]} -gt ${threshhold} ] || [ ${meminfo_D_value[i]} -lt -${threshhold} ];then
                echo "${CURRENT_TIME} outliers : ${meminfo_parameters[i]} was ${meminfo_previous[i]} now ${meminfo_now[i]} D-value = ${meminfo_D_value[i]}" >>$REPORT_LOG_FILE
            fi
            meminfo_previous[i]=${meminfo_now[i]}
        done
    fi
}

mempool_usage_report(){
    if [ $cycles -eq 0 ];then
        for i in $(seq 2 ${mempool_usage_lines});do
            mempool_parameters[i]=`cat /proc/net/lw-conf/mempoolusage | awk '{print $1}'|sed 's/://g' | sed -n ${i}p`> /dev/null
            mempool_blocksize_previous[i]=`cat /proc/net/lw-conf/mempoolusage | awk '{print $2}'| sed -n ${i}p` > /dev/null 
            mempool_blocknum_previous[i]=`cat /proc/net/lw-conf/mempoolusage | awk '{print $3}'| sed -n ${i}p` > /dev/null 
            mempool_totalmem_previous[i]=`expr ${mempool_blocksize_previous[i]} \* ${mempool_blocknum_previous[i]}` > /dev/null 
        done
    else
        echo -------------mempool_usage_report-------------- >> $REPORT_LOG_FILE
        for i in $(seq 2 ${mempool_usage_lines});do
            mempool_blocksize_now[i]=`cat /proc/net/lw-conf/mempoolusage | awk '{print $2}'| sed -n ${i}p` > /dev/null 
            mempool_blocknum_now[i]=`cat /proc/net/lw-conf/mempoolusage | awk '{print $3}'| sed -n ${i}p` > /dev/null 
            mempool_totalmem_now[i]=`expr ${mempool_blocksize_now[i]} \* ${mempool_blocknum_now[i]} `
            mempool_D_value[i]=`expr ${mempool_totalmem_now[i]} - ${mempool_totalmem_previous[i]}`
            if [ ${mempool_D_value[i]} -gt ${threshhold} ] || [ ${mempool_D_value[i]} -lt -${threshhold} ];then
                echo "${CURRENT_TIME} outliers : ${mempool_parameters[i]} was ${mempool_totalmem_previous[i]} now ${mempool_totalmem_now[i]} D-value = ${mempool_D_value[i]}" >>$REPORT_LOG_FILE
            fi
            mempool_totalmem_previous[i]=${mempool_totalmem_now[i]}
        done
     fi
}

mem_usage_report(){
    if [ $cycles -eq 0 ];then
        for i in $(seq 2 ${mem_usage_lines});do
            memusage_parameters[i]=`cat /proc/net/lw-conf/memusage | awk '{print $1}'|sed 's/://g' | sed -n ${i}p`> /dev/null
            memusage_counter_previous[i]=`cat /proc/net/lw-conf/memusage | awk '{print $2}'| sed -n ${i}p` > /dev/null 
            memusage_previous[i]=`cat /proc/net/lw-conf/memusage | awk '{print $3}'| sed -n ${i}p` > /dev/null
        done
    else
        echo -----------------mem_usage_report-------------- >> $REPORT_LOG_FILE
        for i in $(seq 2 ${mem_usage_lines});do
            memusage_counter_now[i]=`cat /proc/net/lw-conf/memusage | awk '{print $2}'| sed -n ${i}p` > /dev/null 
            memusage_now[i]=`cat /proc/net/lw-conf/memusage | awk '{print $3}'| sed -n ${i}p` > /dev/null
            memusage_D_value[i]=`expr ${memusage_now[i]} - ${memusage_previous[i]}`> /dev/null
            if [ ${memusage_D_value[i]} -gt ${threshhold} ] || [ ${memusage_D_value[i]} -lt -${threshhold} ];then
                echo "${CURRENT_TIME} outliers : ${memusage_parameters[i]} was ${memusage_previous[i]} MemCounter ${memusage_counter_previous[i]} now ${memusage_now[i]}  MemCounter ${memusage_counter_now[i]} D-value = ${memusage_D_value[i]}" >>$REPORT_LOG_FILE
            fi
            memusage_counter_previous[i]=${memusage_counter_now[i]}
            memusage_previous[i]=${memusage_now[i]}
        done
    fi
}

vmalloc_report(){
    echo -----------------vmalloc_report-------------- >> $REPORT_LOG_FILE
    for i in $(seq 1 ${mem_info_lines});do
        vmalloc_parameters[i]=`cat /proc/vmallocinfo | awk '{print $3}'| sed -n ${i}p`> /dev/null
        if [ `cat /proc/vmallocinfo | grep ${vmalloc_parameters[i]} | wc -l` -ne 1 ];then
            parameter_exist=0
            if [ $i -gt 0 ];then
                for n in $(seq 0 ${#vmalloc_multiple_parameters[@]});do
                    if [ ${vmalloc_multiple_parameters[n]} ];then
                        if [ ${vmalloc_parameters[i]} == ${vmalloc_multiple_parameters[n]} ];then
                            parameter_exist=1
                            break
                        fi
                    fi
                done
            fi
            if [ ${parameter_exist} -eq 0 ];then
                vmalloc_multiple_parameters[${#vmalloc_multiple_parameters[@]}]=${vmalloc_parameters[i]}
                vmalloc_total[i]=`cat /proc/vmallocinfo | grep ${vmalloc_parameters[i]} | awk '{sum+=$2}END{print sum}'` > /dev/null
                echo "${CURRENT_TIME} ${vmalloc_parameters[i]} : ${vmalloc_total[i]}" >>$REPORT_LOG_FILE
            fi
            continue
        fi
        vmalloc_total[i]=`cat /proc/vmallocinfo | awk '{print $2}'| sed -n ${i}p` > /dev/null 
        echo "${CURRENT_TIME} ${vmalloc_parameters[i]} : ${vmalloc_total[i]}" >>$REPORT_LOG_FILE
    done
    unset vmalloc_multiple_parameters
}

slab_report(){
    if [ $cycles -eq 0 ];then
        for i in $(seq 3 ${slab_info_lines});do
            slab_parameters_previous[i]=`cat /proc/slabinfo | awk '{print $1}'| sed -n ${i}p` > /dev/null
            slab_objsize_previous[i]=`cat /proc/slabinfo | awk '{print $4}'| sed -n ${i}p` > /dev/null 
            slab_objnum_previous[i]=`cat /proc/slabinfo | awk '{print $3}'| sed -n ${i}p` > /dev/null 
            slab_total_previous[i]=`expr ${slab_objsize_previous[i]} \* ${slab_objnum_previous[i]}` > /dev/null 
        done
    else
        echo -----------------slab_report-------------- >> $REPORT_LOG_FILE
        for i in $(seq 3 ${slab_info_lines});do
            slab_parameters_now[i]=`cat /proc/slabinfo | awk '{print $1}'| sed -n ${i}p`> /dev/null
            slab_objsize_now[i]=`cat /proc/slabinfo | awk '{print $4}'| sed -n ${i}p` > /dev/null 
            slab_objnum_now[i]=`cat /proc/slabinfo | awk '{print $3}'| sed -n ${i}p` > /dev/null 
            slab_total_now[i]=`expr ${slab_objnum_now[i]} \* ${slab_objsize_now[i]}` > /dev/null 
            
            for k in $(seq 0 ${#slab_parameters_previous[@]});do
                if [ ${slab_parameters_previous[k]} ];then
                    if [ ${slab_parameters_now[i]} == ${slab_parameters_previous[k]} ];then
                        slab_D_value[i]=`expr ${slab_total_now[i]} - ${slab_total_previous[k]}`
                        if [ ${slab_D_value[i]} -gt ${threshhold} ] || [ ${slab_D_value[i]} -lt -${threshhold} ];then
                            echo "${CURRENT_TIME} outliers : ${slab_parameters_now[i]} was ${slab_total_previous[k]} now ${slab_total_now[i]} D-value = ${slab_D_value[i]}" >>$REPORT_LOG_FILE
                        fi
                    fi
                fi
            done
            slab_total_previous[i]=${slab_total_now[i]}
            slab_parameters_previous[i]=${slab_parameters_now[i]}
        done
     fi
}

memory_usage_collect(){
    if [ ! -d $MOMITOR_PATH ]; then
        mkdir -p $MOMITOR_PATH
    fi
    
    echo ============================== >> $MONITOR_LOG_FILE
    echo ${CURRENT_TIME} >>$MONITOR_LOG_FILE
    
    if [ -f /proc/meminfo ]; then
        echo ============================== >> $MONITOR_LOG_FILE
        echo -------------meminfo-------------- >> $MONITOR_LOG_FILE
        cat /proc/meminfo >> $MONITOR_LOG_FILE 2>&1
    else
        echo **********/proc/meminfo directory does not exist********** >> $MONITOR_LOG_FILE
    fi
    
    if [ -f /proc/net/lw-conf/mempoolusage ]; then
        echo -------------mempoolusage-------------- >> $MONITOR_LOG_FILE
        cat /proc/net/lw-conf/mempoolusage >> $MONITOR_LOG_FILE 2>&1
    else
        echo **********/proc/net/lw-conf/mempoolusage directory does not exist********** >> $MONITOR_LOG_FILE
    fi
    
    if [ -f /proc/net/lw-conf/memusage ]; then
        echo -------------memusage-------------- >> $MONITOR_LOG_FILE
        cat /proc/net/lw-conf/memusage >> $MONITOR_LOG_FILE 2>&1
    else
        echo ============================== >> $MONITOR_LOG_FILE
        echo **********/proc/net/lw-conf/memusage does not exist********** >> $MONITOR_LOG_FILE
    fi
    
    if [ -f /proc/vmallocinfo ]; then
        if [ $vmalloc_flag -eq 1 ];then
            echo -------------vmallocinfo-------------- >> $MONITOR_LOG_FILE
            cat /proc/vmallocinfo >> $MONITOR_LOG_FILE 2>&1
        fi
    else
        echo ============================== >> $MONITOR_LOG_FILE
        echo **********/proc/vmallocinfo does not exist********** >> $MONITOR_LOG_FILE
    fi
    
    if [ -f /proc/slabinfo ]; then
        if [ $slab_flag -eq 1 ];then
            echo -------------slabinfo-------------- >> $MONITOR_LOG_FILE
            cat /proc/slabinfo >> $MONITOR_LOG_FILE 2>&1
        fi
    else
        echo **********/proc/slabinfo does not exist********** >> $MONITOR_LOG_FILE
    fi
}

clean_cache(){
    mem_usaged=`free -m | grep 'Mem' | awk '{print $3}'` > /dev/null
    mem_total=`free -m | grep 'Mem' | awk '{print $2}'` > /dev/null
    mem_cached=`free -m | grep 'Mem' | awk '{print $6}'` > /dev/null
    mem_usaged_total=`expr ${mem_usaged} + ${mem_cached}`
    mem_usage_percent=$(echo "scale=4; ${mem_usaged_total} / ${mem_total} * 100" | bc) > /dev/null
    echo -----------------mem usage percent-------------- >> $REPORT_LOG_FILE
    echo "${CURRENT_TIME} Memory usage [Use：${mem_usaged_total}MB] [Total：${mem_total}MB] [Percent ${mem_usage_percent}%]" >> $REPORT_LOG_FILE
    if [ `echo "${mem_usage_percent} > ${mem_usage_max_percent}" | bc` -eq 1 ];then
        sync && echo 3 > /proc/sys/vm/drop_caches
        mem_usaged=`free -m | grep 'Mem' | awk '{print $3}'` > /dev/null
        mem_total=`free -m | grep 'Mem' | awk '{print $2}'` > /dev/null
        mem_cached=`free -m | grep 'Mem' | awk '{print $6}'` > /dev/null
        mem_usaged_total=`expr ${mem_usaged} + ${mem_cached}`
        mem_usage_percent=$(echo "scale=4; ${mem_usaged_total} / ${mem_total} * 100" | bc) > /dev/null
        echo "${CURRENT_TIME} Memory usage is above ${mem_usage_max_percent}% , cleaning cache" >> $REPORT_LOG_FILE
        echo "${CURRENT_TIME} Memory usage after clean [Use：${mem_usaged_total}MB] [Total：${mem_total}MB] [Percent ${mem_usage_percent}%]" >> $REPORT_LOG_FILE
    fi  
}

[[ -z $1 || $1 == "-help" || $1 == "--help" ]] && {
	help
	exit 1
}

until [ $# -eq 0 ]
do
    if [ $1 == "Start" ];then
        start_flag=1
    fi
    if [ $1 == "Stop" ];then
        stop_flag=1
    fi
    if [ $1 == "EnableVmalloc" ];then
        vmalloc_flag=1
    fi
    if [ $1 == "EnableSlab" ];then
        slab_flag=1
    fi
    if [ $1 == "SetInterval" ];then
        shift
        interval_time=$1
    fi
    if [ $1 == "SetThreshhold" ];then
        shift
        threshhold=$1
    fi
    if [ $1 == "SetPercent" ];then
        shift
        mem_usage_max_percent=$1
    fi
    shift
done

if [ $stop_flag -eq 1 ];then
    stop_script
    exit 1
fi

while true;
do
#time
    CURRENT_TIME=`date +'%y—%m-%d %H:%M:%S'`
    CURRENT_DATE=`date +'%m_%d'`
#file
    MONITOR_LOG_FILE=$MOMITOR_PATH/mem_usage_${CURRENT_DATE}.log
    REPORT_LOG_FILE=$MOMITOR_PATH/mem_report_${CURRENT_DATE}.log
#lines
    mem_info_lines=`cat /proc/meminfo | wc -l`
    mempool_usage_lines=`cat /proc/net/lw-conf/mempoolusage | wc -l`
    mem_usage_lines=`cat /proc/net/lw-conf/memusage | wc -l`
    vmalloc_info_lines=`cat /proc/vmallocinfo | wc -l`
    slab_info_lines=`cat /proc/slabinfo | wc -l`
#log delete
    find ${MOMITOR_PATH} -mtime +6 -name "mem_usage*" -execdir rm -rf {} \;
    find ${MOMITOR_PATH} -mtime +6 -name "mem_report*" -execdir rm -rf {} \;

    memory_usage_collect
    meminfo_report
    mempool_usage_report
    mem_usage_report
    if [ $vmalloc_flag -eq 1 ];then
        vmalloc_report
    fi
    if [ $slab_flag -eq 1 ];then
        slab_report
    fi
    clean_cache
    cycles=`expr $cycles + 1` 
    sleep ${interval_time}m
done;
