#!/bin/sh
#v1.5
#2021.01.15
#2022.01.11  Correspongding engine Id could not be located by ifname since multi-ltt supported, so we simply set pcap on for all engines

collect_top_path=/var/log/monitor/ping_pcap/
script_name=$(basename $0)
g_src_ip=""
g_dst_ip=""
g_interface=""
g_PingCount=100
g_PingTimeout=1
g_PingInterval=0.1
g_PacketLossThresh=10
g_MaxFileNum=10
g_CollectTimes=10

g_dst_ip_hex=""
g_engine_id=""
g_engine_total_num=0
g_ShouldStop=0
g_ColletCount=0
start_flag=0
stop_flag=0
engine_filter_ip_support=0
is_engine_capturing=0

#engine_pcap indicate file
engine_pcap_indicater_file=/tmp/engine_pcap_indicater

help() {
    echo "Usage: $0 <Start/Stop> <SetSrcIp <Ip>> <SetDstIp <Ip>> <SetInterface <Interface>> [SetCollectTimes <Num>]"
    echo "      e.g. $0 Start SetSrcIp xxx.xxx.xxx.xxx SetDstIp xxx.xxx.xxx.xxx SetInterface eth0 SetCollectTimes 10 &"
}

stop_script(){
    for pid in `ps aux | grep "${script_name} Start" | grep -v grep | awk '{print $2}'`
    do
        kill -9 $pid
    done
    for appex in `ls /proc/net/appex* -d`
    do
        echo 0 > $appex/pcapEnable
    done
    wan_pcap_stop
}

iptohex() {
    IFS=.
    for str in $1
    do
        printf "%02X" $str
    done
}

locate_engine_id(){
    if [ -d /proc/net/appex ];then
        engine_total_num=`ls /proc/net/appex* -d | wc -l` > /dev/null 2>&1
    else
        echo "Could not locate engine numbers"
        exit 1
    fi

    local cnt=0
    for engine_num in $(seq 1 ${engine_total_num})
    do
        cnt=$(($cnt+1))
        g_engine_id[$cnt]=$((${engine_num}-1))
    done
}

wan_pcap_start(){
    tcpdump -i $g_interface -s 256 -w ${collect_path}/${g_interface}_`date "+%Y-%m-%d_%H%M%S"`_ping_pcap.pcap > /dev/null 2>&1 &
}

wan_pcap_stop(){
    kill -9 `ps aux | grep "tcpdump -i $g_interface" | grep -v grep | awk '{print $2}'` > /dev/null 2>&1
}

ping_pre_set(){
    local engine_id=$1
    if [ $engine_id == 0 ];then
        echo  "pcapFilterInnerIp: 1" > /proc/net/appex/cmd
        echo  "pcapFilterProto: 1" > /proc/net/appex/cmd
        echo  "pcapFilterIp: ${g_dst_ip_hex}" > /proc/net/appex/cmd
    else
        echo  "pcapFilterInnerIp: 1" > /proc/net/appex$engine_id/cmd
        echo  "pcapFilterProto: 1" > /proc/net/appex$engine_id/cmd
        echo  "pcapFilterIp: ${g_dst_ip_hex}" > /proc/net/appex$engine_id/cmd   
    fi
}

ping_aft_set(){
    local engine_id=$1
    if [ $engine_id == 0 ];then
        echo  "pcapFilterInnerIp: 0" > /proc/net/appex/cmd
        echo  "pcapFilterProto: 0" > /proc/net/appex/cmd
        echo  "pcapFilterIp: " > /proc/net/appex/cmd
    else
        echo  "pcapFilterInnerIp: 0" > /proc/net/appex$engine_id/cmd
        echo  "pcapFilterProto: 0" > /proc/net/appex$engine_id/cmd
        echo  "pcapFilterIp: " > /proc/net/appex$engine_id/cmd   
    fi
}

set_engine_pcap(){
    for engine_num in ${g_engine_id[@]}
    do
        if [ $engine_num -eq 0 ];then
            echo $1 > /proc/net/appex/pcapEnable

        else
            echo $1 > /proc/net/appex$engine_num/pcapEnable
        fi
    done
}

file_delete(){
    local file_num=`find $collect_top_path -name "$1" | wc -l`
    if [ $file_num -gt $g_MaxFileNum ];then
        local del_num=$(($g_MaxFileNum-$file_num))
        find $collect_top_path -name "$1" -type d | xargs ls -dta | tail -n ${del_num} | xargs rm -rf
    fi
}

stats_collect(){
    stats_log_file=${collect_path}/stats_$(date +'%y_%m_%d_%H:%M:%S').log
    echo "===============/proc/net/lw-conf/ltt/stats===============" >> ${stats_log_file}
    cat /proc/net/lw-conf/ltt/stats >> ${stats_log_file} 2>&1
    
    echo "===============/proc/net/lw-conf/lw_stats===============" >> ${stats_log_file}
    cat /proc/net/lw-conf/lw_stats >> ${stats_log_file} 2>&1
    
    for appex in `ls /proc/net/ | grep appex`;do
        echo "===============/proc/net/${appex}/stats===============" >> ${stats_log_file}
        cat /proc/net/${appex}/stats >> ${stats_log_file} 2>&1
    done
}

info_collect()
{
    stats_collect
    cat /proc/net/lw-conf/flow_table | grep $g_dst_ip >> ${collect_path}/flow_${CURRENT_TIME}.log 2>&1
    cat /proc/net/lw-conf/policy_table >> ${collect_path}/policy_${CURRENT_TIME}.log 2>&1
    sleep 2
    stats_collect
}

ping_detect(){
    if [ $engine_filter_ip_support -eq 1 ] && [ $is_engine_capturing -eq 0 ];then
        for engine_num in ${g_engine_id[@]}
        do
            ping_pre_set $engine_num
        done
    fi
    result=`ping -i $g_PingInterval -q -c $g_PingCount -W $g_PingTimeout -I $g_src_ip $g_dst_ip |grep -vE ' bytes of data|^$'`
    echo $result >> $PING_LOG_FILE
    local packet_loss=`cat $PING_LOG_FILE | sed 's/ /\n/g' | grep % | cut -d "%" -f 1`
    if [ $packet_loss -gt $g_PacketLossThresh ];then
        echo "Paclet loss :$packet_loss, script stop" >> $PING_LOG_FILE
        g_ColletCount=$(($g_ColletCount+1))
        if [ $g_ColletCount -eq $g_CollectTimes ];
        then
            g_ShouldStop=1
        else
            info_collect
        fi
    fi
    
    if [ $engine_filter_ip_support -eq 1 ] && [ $is_engine_capturing -eq 0 ];then
        for engine_num in ${g_engine_id[@]}
        do
            ping_aft_set $engine_num
        done
    fi
}

[[ -z $1 || $1 == "-help" || $1 == "--help" ]] && {
    help
    exit 1
}

until [ $# -eq 0 ]
do
    if [ $1 == "Start" ];then
        start_flag=1
    fi
    if [ $1 == "Stop" ];then
        stop_flag=1
    fi
    if [ $1 == "SetSrcIp" ];then
        shift
        g_src_ip=$1
    fi
    if [ $1 == "SetDstIp" ];then
        shift
        g_dst_ip=$1
        g_dst_ip_hex=0x$(iptohex $1)
    fi
    if [ $1 == "SetInterface" ];then
        shift
        g_interface=$1
    fi
    if [ $1 == "SetCollectTimes" ];then
        shift
        g_CollectTimes=$1
    fi
    shift
done

if [ $stop_flag -eq 1 ];then
    stop_script
    exit 1
fi

if [ $start_flag -ne 1 ];then
    exit
fi

[ ! -n "${g_src_ip}" -o ! -n "${g_dst_ip}" -o ! -n "${g_interface}" ] && {
    echo "invalid parameter, please refer to help($0 --help)"
    exit 1
}

locate_engine_id

if [ -n "`grep pcapFilterInnerIp /proc/net/appex/cmd`" ];then
    engine_filter_ip_support=1
fi

is_exist=`ps aux | grep "${script_name} Start" | grep -v grep | wc -l` > /dev/null
if [ $is_exist -gt 2 ];then
    echo "$0 already Running"
    exit 1
fi

while true;
do
    CURRENT_TIME=`date +'%y_%m_%d_%H:%M:%S'`
    collect_path=${collect_top_path}/ping_pcap_${CURRENT_TIME}
    if [ ! -d $collect_path ]; then
        mkdir -p $collect_path
    fi
    PING_LOG_FILE=${collect_path}/ping_${CURRENT_TIME}.log
    
    wan_pcap_start
   
    #To check if engine pcap is in processing by other script
    if [ ! -f $engine_pcap_indicater_file ];then
        set_engine_pcap 1
    fi
    
    ping_detect
    wan_pcap_stop
    
    #To check if engine pcap is in processing by other script
    if [ ! -f $engine_pcap_indicater_file ];then
        set_engine_pcap 0
        for engine_num in ${g_engine_id[@]}
        do
            cp -r /appex/log/$engine_num ${collect_path}/pcap_engine${engine_num}_$CURRENT_TIME
            find /appex/log/$engine_num -maxdepth 1 -type f -execdir rm -- '{}' \;
        done
    fi
    
    if [ $g_ShouldStop -eq 1 ];then
        info_collect
        exit 0
    fi
    file_delete "ping_pcap_*"
done;
