#!/bin/bash
# lw pop smp affinity enhancement 
# 
# 1.2: grep -v virtual 
# 1.1: add irqbalance disable
# 1.0: initial draft 

Version="v1.1"

Supported_Net_Driver="virtio_net mlx5_core hv_netvsc vmxnet3"
Phy_Device=""
Cpu_Count=""

ConfigIrqList()
{
    local irqs="$*"
    local cores=${Cpu_Count}
    local i=0

    let cores=cores-1

    for irq in ${irqs}
    do
        echo ${i} > /proc/irq/${irq}/smp_affinity_list
        echo "### bind irq ${irq} to CPU ${i}"

        if [[ "${i}" == "${cores}" ]]
        then 
            i=0
        else
            let i=i+1
        fi
    done
}

ConfigVirtioInput()
{
    local irqSet=`cat /proc/interrupts  | grep -i ".*virtio.*input.*" | awk -F ':' '{print $1}'`
    echo "### configuring virtio input irq"
    ConfigIrqList "${irqSet}"
}

ConfigVirtioOutput()
{
    local irqSet=`cat /proc/interrupts  | grep -i ".*virtio.*output.*" | awk -F ':' '{print $1}'`
    echo "### configuring virtio output irq"
    ConfigIrqList "${irqSet}"
}

ConfigforVirtio()
{
    ConfigVirtioInput
    ConfigVirtioOutput
}

ConfigforMLX5()
{
    local irqSet=$(cat /proc/interrupts | egrep 'mlx4|mlx5' | awk '{print $1}' | sed 's/://')
    echo "### configuring mlx5 irq"
    ConfigIrqList "${irqSet}"
}

ConfigPhyChannel()
{
    local phy_name=${1}
    local queue_max=$(ethtool -l ${phy_name} | grep Combined | sed -n 1p | awk '{print $2}')
    local queue_set=$(ethtool -l ${phy_name} | grep Combined | sed -n 2p | awk '{print $2}')

    if [[ "${queue_max}" == "1" ]]
    then
        return 
    fi

    if [[ "${queue_set}" != "${queue_max}" ]]
    then
        ethtool -L ${phy_name} combined ${queue_max}
        [ $? -eq 0 ] && {
            echo "### set $phy_name combined ${queue_max}"
        }
    fi
}


#
# vmxnet3 is not like other drivers: virtio_net mlx5_core hv_netvsc
#
# its channels number is fixed value, can not change:
#
#        min(8, Cpu_Count)        # on our online POP, max Cpu_Count is 8.
#
# and its irq naming pattern is:
#
#        ${phy_name}-rxtx
#
#
# so its ConfigforVmxnet3() is not like other drivers, it need to configure irq list for each
# interface.
#

ConfigforVmxnet3()
{
    local phy_name=${1}

    local irqSet=`cat /proc/interrupts | grep ${phy_name}-rxtx | awk '{print $1}' | sed 's/://g'`

    echo "### configuring vmxnet3 ${phy_name} irq"	

    ConfigIrqList "${irqSet}"
}


CheckCpuNum()
{
    local cpu_num=`cat /proc/cpuinfo |grep "processor"|sort -u|wc -l` > /dev/null
    if [ ${cpu_num} -eq 1 ];then
        exit 0
    else
        Cpu_Count=${cpu_num}
    fi
}


GetAllPhyDevice()
{
    local net_device=$(ls /sys/class/net)
    Phy_Device=""
    
    for device in ${net_device}
    do
        if [ ! -d /sys/class/net/${device}/device ];then
            continue
        fi
        Phy_Device=$(echo -e "${device}\n${Phy_Device}")
    done
}

StopIrqbalance()
{
    local exist=`ps aux | grep irqbalance | grep -v grep`
    if [ -n "${exist}" ]
    then
        systemctl stop irqbalance
        systemctl disable irqbalance
    fi
}


Start()
{
    local is_virtio="0"
    local is_mlx5="0"

    echo "POP IRQ script ${Version}"

    StopIrqbalance

    CheckCpuNum

    GetAllPhyDevice

    for device in ${Phy_Device}
    do
        driver=$(ethtool -i ${device} | grep driver | awk '{print $2}')
        case ${driver} in
            "virtio_net")
                is_virtio="1"
                ConfigPhyChannel ${device}
                ;;
            "mlx5_core")
                is_mlx5="1"
                ConfigPhyChannel ${device}
                ;;
            "hv_netvsc")
                ConfigPhyChannel ${device}
                ;;
            "vmxnet3")
                ConfigforVmxnet3 ${device}
                ;;
        esac
    done

    if [[ "${is_virtio}" == "1" ]]
    then
        ConfigforVirtio
    fi

    if [[ "${is_mlx5}" == "1" ]]
    then
        ConfigforMLX5
    fi

}

Start
