#!/bin/bash
#v2.0
#2021.03.31
Monitor_Path=/var/log/monitor/pppoe_monitor
script_name=$(basename $0)

declare -a pppoe_phyif
declare -a pppoe_pppname

start_flag=0
stop_flag=0
sleep_time=10
max_pcap_num=50
enable_delay=0

ping_target=(
    61.135.169.121
    111.206.176.91
    47.95.164.112
)

help() {
    echo "Usage: $0 <Start/Stop> [SetInterval  <Time>] [EnableDelay]"
    echo "      e.g. $0 Start SetInterval 10 EnableDelay &"
    echo "NOTE: Interval in unit of minute. Default time is 10 second if no interval is specified."
    echo "      EnableDelay: wait 120 second to collect info. Default is disable"
}

stop_script(){
    for pid in `ps aux | grep "$script_name Start" | grep -v grep | awk '{print $2}'`
    do
        kill -9 $pid
    done
}

#ping_test pppoe_pppname ping_target
ping_test(){
    ping -I $1 -c 5 -w 5 $2 > /dev/null 2>&1
    if [ $? -ne 0 ];then
        let $3=$3+1
    fi
}

pre_set(){
    for i in $(seq 0 10);do
        last_trigger_time[i]=0
    done

    pppoe_list=`cat /proc/net/lw-conf/if-tbl | grep "Name: ppp" | cut -d ":" -f 2 | sed 's/ //g'` > /dev/null
    pppoe_count=0

    for pppoe in $pppoe_list;do
        state=`cat /proc/net/lw-conf/if-tbl | grep -A 15 "Name: ${pppoe}" | grep "State: 0"`> /dev/null
        if [ -n "$state" ];then
            eval pppoe_pppname[$pppoe_count]=${pppoe}
            eval pppoe_phyif[$pppoe_count]=`cat /proc/net/lw-conf/if-tbl | grep -B 15 "PPPoEIf: ${pppoe}" | grep "Name:" | cut -d ":" -f 2 | sed 's/ //g'` > /dev/null
            pppoe_count=`expr ${pppoe_count} + 1`
        fi
    done
}

#ppp_restart pppoe_phyif
ppp_restart(){
    for pid in `ps aux | grep "pppd call" | grep "$1" | awk '{print $2}'`
    do
        kill -9 $pid
    done
    pppd call $1 & >> ${log_file}
    sleep 60
}

[[ $1 == "-help" || $1 == "--help" ]] && {
    help
    exit 1
}

until [ $# -eq 0 ]
do
    if [ $1 == "Start" ];then
        start_flag=1
    fi
    if [ $1 == "Stop" ];then
        stop_flag=1
    fi
    if [ $1 == "SetInterval" ];then
        shift
        sleep_time=$1
    fi
    if [ $1 == "EnableDelay" ];then
        enable_delay=1
    fi
    shift
done

if [ $stop_flag -eq 1 ];then
    stop_script
    exit 1
fi

if [ $start_flag -ne 1 ];then
    help
    exit 1
fi

is_exist=`ps aux | grep ${script_name}| grep -v grep | wc -l` > /dev/null
if [ $is_exist -gt 2 ];then
    echo "$0 already Running"
    exit 1
fi

if [ ${enable_delay} -eq 1 ];then
    sleep 120
fi

if [ ! -d ${Monitor_Path} ];then
    mkdir -p $Monitor_Path
fi

pre_set

while true
do
    log_file=${Monitor_Path}/pppoe_monitor_`date "+%Y-%m-%d"`.log 
    pcap_file_num=`find ${Monitor_Path} -type f -name "*_pppoe_monitor.pcap" | wc -l`
    if [ $pcap_file_num -gt $max_pcap_num ];then
        pcap_del_file_num=`expr $pcap_file_num - $max_pcap_num`
        find ${Monitor_Path} -type f -name "*_pppoe_monitor.pcap" | xargs ls -dta | tail -n ${pcap_del_file_num} | xargs rm -rf
    fi
    
    find ${Monitor_Path} -type f -name "pppoe_monitor_*" -mtime +7 -execdir rm -- '{}' \;
    
    time_now=`date +%s`
    for((i=0;i<${#pppoe_phyif[@]};i++));do
        fail_times=0
        exist=`ps aux | grep pppd | grep ${pppoe_phyif[i]} | grep -v grep` > /dev/null
        if [ -n "${exist}" ];then
            echo "`date "+%Y-%m-%d_%H%M"` ${pppoe_phyif[i]} present" >> ${log_file}
        else
            echo "`date "+%Y-%m-%d_%H%M"` pppd process for ${pppoe_phyif[i]} absent, restart pppd call" >> ${log_file}
            ppp_restart ${pppoe_phyif[i]}
        fi
        
        for target in ${ping_target[@]};do
            ping_test ${pppoe_pppname[${i}]} ${target} fail_times
        done
        
        if [ $fail_times -eq ${#ping_target[@]} ];then
            if [ $time_now -gt ${last_trigger_time[i]} ];then
                last_trigger_time[i]=`expr $time_now + 600`
                tcpdump -i ${pppoe_phyif[i]} -w ${Monitor_Path}/${pppoe_phyif[i]}_`date "+%Y-%m-%d_%H%M"`_pppoe_monitor.pcap > /dev/null 2>&1 &
                sleep 5 
                kill -9 `ps aux | grep "tcpdump -i ${pppoe_phyif[i]}" | grep -v grep | awk '{print $2}'` > /dev/null 2>&1
            fi
            echo "`date "+%Y-%m-%d_%H%M"` ${pppoe_phyif[i]} ping test fialed ,restart pppd call" >> ${log_file}
            ppp_restart ${pppoe_phyif[i]}
        fi
    done
    sleep $sleep_time
done
