#!/bin/sh
#v1.8
#2020.11.18
COLLECT_PATH=/var/log/monitor/route_collect
script_name=$(basename $0)

g_max_entry_num=505
conf_file=/etc/cpeagent/cpeagent.conf
start_flag=0
stop_flag=0
g_interval=60
g_max_disk_usage=2048
g_filter_ip=""
g_enable_ipset=0
g_enable_flow=0

help() {
    echo "Usage: $0 <Start/Stop> <SetFilterIp <Ip>> [SetInterval <Time(s)>] [SetMaxEntryNum <Num>]  [SetMaxDiskUsage <MB>] [EnableIpset] [EnableFlow]"
    echo "      e.g. $0 Start SetFilterIp xxx.xxx.xxx.xxx SetInterval 60 SetMaxEntryNum 500 SetMaxDiskUsage 2048 EnableIpset EnableFlow &"
    echo "      SetFilterIp: specifc ip for flow collection. Default MaxEntryNum is 500"
    echo "NOTE: Interval in unit of second. Default time is 60 seconds if no interval is specified."
    echo "      SetMaxEntryNum: Collection entry limits for a single route. Default MaxEntryNum is 500"
    echo "      Max disk usage to limit collect info. Default max disk usage is 2048MB(2G)"
    echo "      EnableIpset: to collect ipset info"
    echo "      EnableFlow: to collect flow info"
}

stop_script(){
    for pid in `ps aux | grep "$script_name Start" | grep -v grep | awk '{print $2}'`
    do
        kill -9 $pid
    done
}

route_collect(){
    echo "===============ip rule===============" >> ${LOG_FILE}
    ip rule >> ${LOG_FILE} 2>&1
    echo "===============ip route show===============" >> ${LOG_FILE}
    ip route show >> ${LOG_FILE} 2>&1

    ip rule | while read line
    do
        id=`echo "$line" | awk '{print $1}' | sed 's/://'` > /dev/null
        echo "================ip route show table $id==================" >> ${LOG_FILE}
        ip route show table $id | head -n ${g_max_entry_num} >> ${LOG_FILE} 2>&1
    done

    if [ $g_enable_ipset -eq 1 ];then
        echo "===============LPMPolicy===============" >> ${LOG_FILE}
        head -n ${g_max_entry_num} /proc/net/lw-conf/ipset-RDOnly/LPMPolicy >> ${LOG_FILE}
        echo "===============super ipset ${customer_id}_${cpe_id}===============" >> ${LOG_FILE}
        head -n ${g_max_entry_num} /proc/net/lw-conf/ipset-RDOnly/${customer_id}_${cpe_id} >> ${LOG_FILE} 2>&1

        for sub_ipset in `ls /proc/net/lw-conf/ipset-RDOnly/lan*`;do
            echo "===============${sub_ipset}===============" >> ${LOG_FILE}
            head -n ${g_max_entry_num} ${sub_ipset} >> ${LOG_FILE} 2>&1
        done

        for peer_lan_subnet in `ls /proc/net/lw-conf/ipset-RDOnly/super_*`;do
            peer_id=`echo "${peer_lan_subnet}" | awk -F ''$customer_id'_' '{print $2}'` > /dev/null 
            if [ -n ${peer_id} ];then
                echo "===============${peer_lan_subnet}===============" >> ${LOG_FILE}
                head -n ${g_max_entry_num} ${peer_lan_subnet} >> ${LOG_FILE} 2>&1
                for peer_sub_ipset in `ls /proc/net/lw-conf/ipset-RDOnly/cpe* | grep ${peer_id}`;do
                    echo "===============${peer_sub_ipset}===============" >> ${LOG_FILE}
                    head -n ${g_max_entry_num} ${peer_sub_ipset} >> ${LOG_FILE} 2>&1
                done
            fi
        done
    fi
    
    echo "===============/proc/net/lw-conf/ltt/stats===============" >> ${LOG_FILE}
    cat /proc/net/lw-conf/ltt/stats >> ${LOG_FILE} 2>&1
    
    echo "===============/proc/net/lw-conf/lw_stats===============" >> ${LOG_FILE}
    cat /proc/net/lw-conf/lw_stats >> ${LOG_FILE} 2>&1
    
    for appex in `ls /proc/net/ | grep appex`;do
        echo "===============/proc/net/${appex}/stats===============" >> ${LOG_FILE}
        cat /proc/net/${appex}/stats >> ${LOG_FILE} 2>&1
    done
    
    if [ $g_enable_flow -eq 1 ];then
        cat /proc/net/lw-conf/flow_table | grep $g_filter_ip >> ${FLOW_LOG_FILE} 2>&1
    fi
    cat /proc/net/lw-conf/policy_table >> ${POLICY_LOG_FILE} 2>&1
}

[[ -z $1 || $1 == "-help" || $1 == "--help" ]] && {
    help
    exit 1
}

until [ $# -eq 0 ]
do
    if [ $1 == "Start" ];then
        start_flag=1
    fi
    if [ $1 == "Stop" ];then
        stop_flag=1
    fi
    if [ $1 == "SetInterval" ];then
        shift
        g_interval=$1
    fi
    if [ $1 == "SetFilterIp" ];then
        shift
        g_filter_ip=$1
    fi
    if [ $1 == "SetMaxEntryNum" ];then
        shift
        g_max_entry_num=$1
    fi
    if [ $1 == "SetMaxDiskUsage" ];then
        shift
        g_max_disk_usage=`expr ${1} \* 1024`
    fi
    if [ $1 == "EnableIpset" ];then
        g_enable_ipset=1
    fi
    if [ $1 == "EnableFlow" ];then
        g_enable_flow=1
    fi
    shift
done

if [ $stop_flag -eq 1 ];then
    stop_script
    exit 1
fi

if [ $start_flag -ne 1 ];then
    exit
fi

is_exist=`ps aux | grep "$script_name Start" | grep -v grep | wc -l` > /dev/null
if [ $is_exist -gt 2 ];then
    echo "$0 already Running"
    exit 1
fi

[ ! -d /proc/net/lw-conf/ipset-RDOnly ] && {
    echo "/proc/net/lw-conf/ipset-RDOnly Not Found"
    exit 1
}

if [ ! -n "$g_filter_ip" ];then
    echo "ip not set"
    exit 1
fi

if [ ! -d $COLLECT_PATH ]; then
    mkdir -p $COLLECT_PATH
fi

customer_id=`cat ${conf_file} | grep customerid | cut -d \= -f 2`
cpe_id=`cat ${conf_file} | grep cpeid | cut -d \= -f 2`

while true;
do
    CURRENT_TIME=`date +'%y_%m_%d_%H:%M:%S'`
    LOG_FILE=${COLLECT_PATH}/route_collect_${CURRENT_TIME}.log
    FLOW_LOG_FILE=${COLLECT_PATH}/route_collect_flow_${CURRENT_TIME}.log
    POLICY_LOG_FILE=${COLLECT_PATH}/route_collect_policy_${CURRENT_TIME}.log
    route_collect
    log_size=`du -s ${COLLECT_PATH} | awk '{print $1}'`
    if [ ${log_size} -gt ${g_max_disk_usage} ];then
        total_file_num=`ls ${COLLECT_PATH} | wc -l`
        del_num=`expr $total_file_num / 2`
        find ${COLLECT_PATH} -name "*route_collect_*" -type f | xargs ls -dta | tail -n ${del_num} | xargs rm -rf
    fi
    sleep ${g_interval}
done;
