#!/bin/sh
#v1.0.5
#2021.01.21
COLLECT_PATH=/var/log/monitor/session_monitor
script_name=$(basename $0)
g_interval=60
g_sessions_threshhold=2000
g_enable_pcap=0
g_collect_times=0
g_cpu_thresh=80
g_engine_id=0
g_interface=""
g_engine_path=""
g_last_check_time=0
collect_times_mode=0
start_flag=0
stop_flag=0
last_trigger_time=0
collect_cnt=0
g_cpu_id=""
g_cpu_usage=0

#flow will not be collected if disk usage of collected info over g_max_disk_usage
g_max_disk_usage=2048

help() {
    echo "Usage: $0 <Start/Stop> [SetInterval <Time(s)>] [SetThreshhold <Num>] [SetMaxDiskUsage <MB>] [SetCollectTimes <Nums>] [EnableEnginePcap <Ifname>]"
    echo "      e.g. $0 Start SetInterval 60 SetThreshhold 2000 SetMaxDiskUsage 2048 SetCollectTimes 1 EnableEnginePcap eth0 &"
    echo "NOTE: Interval in unit of second. Default time is 60 seconds if no interval is specified."
    echo "      Sessions threshhold to collect info. Default sessions threshhold is 2048"
    echo "      Max disk usage to limit flow collection. Default max disk usage is 2048MB(2G), this set only limit flow collection"
    echo "      SetCollectTimes collect times. Default is 0 for continuously collect until max disk usage is reached"
    echo "      EnableEnginePcap to enable engine pcap, should be enabled with SetCollectTimes in case of massive disk usage"
}

stop_script(){
    for pid in `ps aux | grep "${script_name} Start" | grep -v grep | awk '{print $2}'`
    do
        kill -9 $pid
    done
    for appex in `ls /proc/net/appex* -d`
    do
        echo 0 > $appex/pcapEnable
    done
}

locate_engine_id(){
    version=`cat /etc/cpeagent/RPM_version | grep 3.2`
    if [ -n "$version" ];then
        g_engine_id=`cat /proc/net/lw-conf/wan-tbl | grep $g_interface | awk '{print $16}' | cut -d "(" -f 2 | cut -d ")" -f 1 | sed 's/,/\n/g' | sed -n 1p`
    else
        g_engine_id=`cat /proc/net/lw-conf/wan-tbl | grep -A 20 $g_interface | grep EngineNum | cut -d "(" -f 2 | cut -d ")" -f 1 | sed 's/,/\n/g' | sed -n 1p`
    fi
    
    [ ! -n "$g_engine_id" ] && { 
        echo "Could not locate engine Ids"
        exit 1
    }
    if [ $g_engine_id -eq 0 ];then
        g_engine_path=/proc/net/appex
    else
        g_engine_path=/proc/net/appex$g_engine_id
    fi
}

locate_cpu_id(){
    g_cpu_id=`cat $g_engine_path/stats | sed 's/ /\n/g' | grep curCpu | cut -d "=" -f 2 | sed 's/,//g'`
    if [ ! -n "$g_cpu_id" ];then
        echo "Could not locate cpu id"
        exit 1
    fi
}

get_cpu_usage(){
    local stats_pre=`awk '$0 ~/cpu['$1']/' /proc/stat`
    local cpu_idle_pre=`echo $stats_pre | awk '{print $5}'`
    local cpu_total_pre=`echo $stats_pre | awk '{total=$2+$3+$4+$5+$6+$7+$8;printf total}'`
    sleep 2
    local stats_after=`awk '$0 ~/cpu['$1']/' /proc/stat`
    local cpu_idle_after=`echo $stats_after | awk '{print $5}'`
    local cpu_total_after=`echo $stats_after | awk '{total=$2+$3+$4+$5+$6+$7+$8;printf total}'`
    local cpu_total_diff=$(($cpu_total_after-$cpu_total_pre))
    local cpu_idle_diff=$(($cpu_idle_after-$cpu_idle_pre))
    g_cpu_usage=`echo $cpu_idle_diff $cpu_total_diff |awk '{printf("%.0f",(1 - $1/$2)*100)}'` 
}

check_sessions()
{
    local doubled_sessions=`head -n 20 /proc/net/lw-conf/flow_table | grep "Flow Number:" | awk -F "Flow Number: " '{print $2}'` > /dev/null
    [ -z "$doubled_sessions" ] && return
    
    local time_now=`date +%s`
    local sessions=`expr $doubled_sessions / 2`
    echo "$CURRENT_TIME Total sessions: $sessions" >> $SESSION_LOG_FILE
    if [ $sessions -gt $g_sessions_threshhold -a $time_now -gt $last_trigger_time ];then
        local flow_log_size=`du -s ${COLLECT_PATH} | awk '{print $1}'`
        if [ ${flow_log_size} -lt ${g_max_disk_usage} ];then
            if [ $g_enable_pcap -eq 1 ];then
                locate_engine_id
                locate_cpu_id
                get_cpu_usage $g_cpu_id
                if [ $g_cpu_usage -lt $g_cpu_thresh ];then
                    echo "$CURRENT_TIME" >> $FLOW_LOG_FILE
                    cat /proc/net/lw-conf/flow_table >> $FLOW_LOG_FILE
                    if [ $g_enable_pcap -eq 1 ];then
                            echo 1 >  $g_engine_path/pcapEnable
                            sleep 10 && echo 0 > $g_engine_path/pcapEnable && cp -r /appex/log/$g_engine_id ${COLLECT_PATH}/pcap_engine${g_engine_id}_$(date +%Y%m%d_%H%M%S) &
                    fi
                    if [ $collect_times_mode -eq 1 ];then
                        collect_cnt=$(($collect_cnt+1))
                        if [ $g_collect_times -gt 0 -a $collect_cnt -ge $g_collect_times ];then
                            exit 0
                        fi
                    fi
                fi
            else
                echo "$CURRENT_TIME" >> $FLOW_LOG_FILE
                cat /proc/net/lw-conf/flow_table >> $FLOW_LOG_FILE
            fi
        else
            echo "$CURRENT_TIME [flow collected ${flow_log_size} over ${g_max_disk_usage}], disable collecting temporarily" >> $FLOW_LOG_FILE
        fi
        last_trigger_time=`expr $time_now + 600`
    fi
}

[[ -z $1 || $1 == "-help" || $1 == "--help" ]] && {
    help
    exit 1
}

until [ $# -eq 0 ]
do
    if [ $1 == "Start" ];then
        start_flag=1
    fi
    if [ $1 == "Stop" ];then
        stop_flag=1
    fi
    if [ $1 == "SetInterval" ];then
        shift
        g_interval=$1
    fi
    if [ $1 == "SetThreshhold" ];then
        shift
        g_sessions_threshhold=$1
    fi
    if [ $1 == "SetMaxDiskUsage" ];then
        shift
        g_max_disk_usage=`expr ${1} \* 1024`
    fi
    if [ $1 == "SetCollectTimes" ];then
        shift
        collect_times_mode=1
        g_collect_times=$1
    fi
    if [ $1 == "EnableEnginePcap" ];then
        g_enable_pcap=1
        shift
        g_interface=$1
    fi
    shift
done

if [ $stop_flag -eq 1 -o $start_flag -ne 1 ];then
    stop_script
    exit 1
fi

if [ $start_flag -ne 1 ];then
    exit
fi

is_exist=`ps aux | grep "${script_name} Start" | grep -v grep | wc -l` > /dev/null
if [ $is_exist -gt 2 ];then
    echo "$0 already Running"
    exit 1
fi

if [ $g_enable_pcap -eq 1 ];then
    locate_engine_id
    locate_cpu_id
fi

if [ ! -d $COLLECT_PATH ]; then
    mkdir -p $COLLECT_PATH
fi

while true;
do
#time
    CURRENT_TIME=`date +'%y—%m-%d %H:%M:%S'`
    CURRENT_DATE=`date +'%m_%d'`
#file
    FLOW_LOG_FILE=${COLLECT_PATH}/flow_record_${CURRENT_DATE}.log
    SESSION_LOG_FILE=${COLLECT_PATH}/session_report_${CURRENT_DATE}.log
    touch $FLOW_LOG_FILE
    touch $SESSION_LOG_FILE

    check_sessions

    #log delete
    time_now=`date +%s`
    if [ ${time_now} -gt ${g_last_check_time} ];then
        #check every 12hrs
        g_last_check_time=`expr $time_now + 43200`
        find ${COLLECT_PATH} -maxdepth 1 -type f -mtime +7 -name "flow_record_*" -execdir rm -rf {} \;
        find ${COLLECT_PATH} -maxdepth 1 -type f -mtime +7 -name "session_report_*" -execdir rm -rf {} \;
    fi
    sleep ${g_interval}
done;


