#!/bin/sh

####################################################################
# create dynamic route(zebra/bgpd/ospfd/ospf6d) config file for VPN namespace
# $1: VPN-ID
####################################################################

VPNID=$1
[ -z "$VPNID" ] && {
    logger -t "createDynRtConf" "Error: must set VPN-ID(${VPNID})"
    exit 1
}

VPN_PREFIX="ns${VPNID}_"
CONF_DIR=/etc/quagga

PUBLIC_ZEBRA_CONF=${CONF_DIR}/zebra.conf
PUBLIC_BGPD_CONF=${CONF_DIR}/bgpd.conf
PUBLIC_OSPFD_CONF=${CONF_DIR}/ospfd.conf
PUBLIC_OSPF6D_CONF=${CONF_DIR}/ospf6d.conf

ZEBRA_CONF=${CONF_DIR}/${VPN_PREFIX}zebra.conf
BGPD_CONF=${CONF_DIR}/${VPN_PREFIX}bgpd.conf
OSPFD_CONF=${CONF_DIR}/${VPN_PREFIX}ospfd.conf
OSPF6D_CONF=${CONF_DIR}/${VPN_PREFIX}ospf6d.conf

cat > ${ZEBRA_CONF} <<EOF
$(cat ${PUBLIC_ZEBRA_CONF})
log vpn-id $VPNID
EOF

cat > ${BGPD_CONF} <<EOF
$(cat ${PUBLIC_BGPD_CONF})
log vpn-id $VPNID
EOF

cat > ${OSPFD_CONF} <<EOF
$(cat ${PUBLIC_OSPFD_CONF})
log vpn-id $VPNID
EOF

cat > ${OSPF6D_CONF} <<EOF
$(cat ${PUBLIC_OSPF6D_CONF})
log vpn-id $VPNID
EOF

chmod 777 ${ZEBRA_CONF}
chmod 777 ${BGPD_CONF}
chmod 777 ${OSPFD_CONF}
chmod 777 ${OSPF6D_CONF}

chown quagga:quagga ${ZEBRA_CONF}
chown quagga:quagga ${BGPD_CONF}
chown quagga:quagga ${OSPFD_CONF}
chown quagga:quagga ${OSPF6D_CONF}

exit 0
