#!/bin/sh

####################################################################
# register dynamic route service(zebra/bgpd/ospfd) to VPN namespace
# $1: VPN-ID
# $1: VPN-NAME
####################################################################

VPNID=$1
VPNNAME=$2
[ -z "$VPNID" -o -z "${VPNNAME}" ] && {
    logger -t "regDynRtServ" "Error: must set VPN-ID(${VPNID}) and VPN-NAME(${VPNNAME})"
    exit 1
}

VPN_PREFIX="ns${VPNID}_"
CONF_DIR=/etc/quagga
RUN_DIR=/var/run/quagga
SYSTEMD_DIR=/usr/lib/systemd/system

#zebra
ZEBRA_CONF=${CONF_DIR}/${VPN_PREFIX}zebra.conf
ZEBRA_PID=${RUN_DIR}/${VPN_PREFIX}zebra.pid
ZEBRA_VTY=${RUN_DIR}/${VPN_PREFIX}zebra.vty
ZSERV_API=${RUN_DIR}/${VPN_PREFIX}zserv.api
ZEBRA_SERVICE_NAME=${VPN_PREFIX}zebra.service
ZEBRA_SERVICE=${SYSTEMD_DIR}/${ZEBRA_SERVICE_NAME}

#bgpd
BGPD_CONF=${CONF_DIR}/${VPN_PREFIX}bgpd.conf
BGPD_PID=${RUN_DIR}/${VPN_PREFIX}bgpd.pid
BGPD_VTY=${RUN_DIR}/${VPN_PREFIX}bgpd.vty
BGPD_SERVICE_NAME=${VPN_PREFIX}bgpd.service
BGPD_SERVICE=${SYSTEMD_DIR}/${BGPD_SERVICE_NAME}

#ospfd
OSPFD_CONF=${CONF_DIR}/${VPN_PREFIX}ospfd.conf
OSPFD_PID=${RUN_DIR}/${VPN_PREFIX}ospfd.pid
OSPFD_VTY=${RUN_DIR}/${VPN_PREFIX}ospfd.vty
OSPFD_SERVICE_NAME=${VPN_PREFIX}ospfd.service
OSPFD_SERVICE=${SYSTEMD_DIR}/${OSPFD_SERVICE_NAME}

#ospf6d
OSPF6D_CONF=${CONF_DIR}/${VPN_PREFIX}ospf6d.conf
OSPF6D_PID=${RUN_DIR}/${VPN_PREFIX}ospf6d.pid
OSPF6D_VTY=${RUN_DIR}/${VPN_PREFIX}ospf6d.vty
OSPF6D_SERVICE_NAME=${VPN_PREFIX}ospf6d.service
OSPF6D_SERVICE=${SYSTEMD_DIR}/${OSPF6D_SERVICE_NAME}

cat > ${ZEBRA_SERVICE} <<EOF
[Unit]
Description=GNU Zebra routing manager
Wants=network.target
Before=network.target
ConditionPathExists=${ZEBRA_CONF}

[Service]
Type=forking
PIDFile=${ZEBRA_PID}
ExecStartPre=/sbin/ip netns exec ${VPNNAME} /sbin/ip route flush proto zebra table 231
ExecStart=/sbin/ip netns exec ${VPNNAME} /usr/sbin/zebra -d -f ${ZEBRA_CONF} -i ${ZEBRA_PID} -z ${ZSERV_API} -y ${ZEBRA_VTY}
Restart=on-abort

[Install]
WantedBy=multi-user.target
EOF

cat > ${BGPD_SERVICE} <<EOF
[Unit]
Description=BGP routing daemon
BindsTo=${ZEBRA_SERVICE_NAME}
After=${ZEBRA_SERVICE_NAME}
ConditionPathExists=${BGPD_CONF}

[Service]
Type=forking
PIDFile=${BGPD_PID}
ExecStart=/sbin/ip netns exec ${VPNNAME} /usr/sbin/bgpd -d -f ${BGPD_CONF} -i ${BGPD_PID} -z ${ZSERV_API} -y ${BGPD_VTY}
Restart=on-abort

[Install]
WantedBy=multi-user.target
EOF

cat > ${OSPFD_SERVICE} <<EOF
[Unit]
Description=OSPF routing daemon
BindsTo=${ZEBRA_SERVICE_NAME}
After=${ZEBRA_SERVICE_NAME}
ConditionPathExists=${OSPFD_CONF}

[Service]
Type=forking
PIDFile=${OSPFD_PID}
ExecStart=/sbin/ip netns exec ${VPNNAME} /usr/sbin/ospfd -d -f ${OSPFD_CONF} -i ${OSPFD_PID} -z ${ZSERV_API} -y ${OSPFD_VTY}
Restart=on-abort

[Install]
WantedBy=multi-user.target
EOF

cat > ${OSPF6D_SERVICE} <<EOF
[Unit]
Description=OSPF6 routing daemon
BindsTo=${ZEBRA_SERVICE_NAME}
After=${ZEBRA_SERVICE_NAME}
ConditionPathExists=${OSPF6D_CONF}

[Service]
Type=forking
PIDFile=${OSPF6D_PID}
ExecStart=/sbin/ip netns exec ${VPNNAME} /usr/sbin/ospf6d -d -f ${OSPF6D_CONF} -i ${OSPF6D_PID} -z ${ZSERV_API} -y ${OSPF6D_VTY}
Restart=on-abort

[Install]
WantedBy=multi-user.target
EOF

chmod 644 ${ZEBRA_SERVICE} ${BGPD_SERVICE} ${OSPFD_SERVICE} ${OSPF6D_SERVICE}

logger -t "regDynRtServ" "register ${ZEBRA_SERVICE_NAME}/${BGPD_SERVICE_NAME}/${OSPFD_SERVICE_NAME}/${OSPF6D_SERVICE_NAME} to namespace(${VPNNAME})"

exit 0
